/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include "nSX2.h"
#include "PS2Edefs.h"

_u64 nb_dma_transfert = 0;

/*
Transfer  n ( Ps2Bios.DmaC[ DMA_CH2 ].size  ) QuadWord from
Address ( Ps2Bios.DmaC[ DMA_CH2 ].memAdd ) to the GS plugin
*/
void DMA_GS_Transfer_Normal_Mode( void ) {
	_u32 	*DmaAdd;
	char DmaTempText[256];
	
	/*Test if Add is inside Main Memory	*/
	if ( Ps2Bios.DmaC[ DMA_CH2 ].memAdd < 0x01FFFFFF ) {
	
		/*Find the Adress Pointer and pass it to the GS Pluggin*/
		DmaAdd = (  _u32 * )	&MipsMemory [ Ps2Bios.DmaC[ DMA_CH2 ].memAdd ];	
		(*GSgifTransfer)( DmaAdd, (_u32 ) Ps2Bios.DmaC[ DMA_CH2 ].size) ;		

	}
	else {
		if ( debug_dma == 1 )	PrintToDmaOutput (  "GS:Trans.Norm.:MemoryAddr Out of Bound\n" );
	}
	

}



void DMA_GS_Transfer_SourceChain_Mode( void ) {

	_u32 	*DmaAdd;
	char 	DmaTempText[256];

	_u128 	temp;

	_u16    Tag_ID_FLAG;
    _u8     DChainTransfertCompleted;
 
	DMA_GS_Transfer_Normal_Mode();				
     
	/*Chained Mode Initialisation*/
	DChainTransfertCompleted = FALSE;
	
	/*Transfert until end tag*/
	while ( DChainTransfertCompleted != TRUE ) {
		
		/*Read Tag*/
		if ( debug_dma == 1 ) {
  			sprintf(DmaTempText,"GS:TAG Add: %8.8lx\n",Ps2Bios.DmaC[ DMA_CH2 ].tag);
     		PrintToDmaOutput (  DmaTempText );
  			
		}
	    MipsMemoryRead ( &temp  , DATA_DW , Ps2Bios.DmaC[ DMA_CH2 ].tag , 0 ) ;

		if ( debug_dma == 1 ) {
  			sprintf(DmaTempText,"GS:TAG Value: - %16.16llx \n",( long long)temp.lo);
     		PrintToDmaOutput (  DmaTempText );
  			
		}


	
		/*Decode Tag*/
	    Ps2Bios.DmaC[ DMA_CH2 ].size         = temp.lo   & 0xFFFF;      						
	    Tag_ID_FLAG                          = ( temp.lo >> 28 ) & 0x7  ;   
	    Ps2Bios.DmaC[ DMA_CH2 ].memAdd       = (_u32)(( temp.lo >> 32 )& 0xFFFFFFFF); 

		if ( debug_dma == 1 ) {
  			sprintf(DmaTempText,"GS:TAG: ID:%d, s:%d, Addr:%8.8lx\n",Tag_ID_FLAG,Ps2Bios.DmaC[ DMA_CH2 ].size,(long)Ps2Bios.DmaC[ DMA_CH2 ].memAdd);
     		PrintToDmaOutput (  DmaTempText );
  			
		}
	 
	 	/*Execute Tag*/
	    switch ( Tag_ID_FLAG ) {
	    	/*
	    	Transfers the QWC qword from the ADDR field, clears the Dn_CHCR_STR to 0, and ends transfer
	    	#00402790
      		*/
	        case DMA_SRC_CHAIN_MODE_REFE : {
	        	if ( debug_dma == 1 ) PrintToDmaOutput (  "GS:Trans.Chain.:REFE\n" );
 
				DMA_GS_Transfer_Normal_Mode();	

                /*End Chained Transfer*/                                 	        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }

			/*
			Transfers the QWC qword following the tag and reads the succeeding qwords as the next tag
			*/
	        case DMA_SRC_CHAIN_MODE_CONT : {
	        	if ( debug_dma == 1 ) PrintToDmaOutput (  "GS:Trans.Chain.:CNT\n" );
	        	
	        	/*Compute Address after the Tag*/
	        	Ps2Bios.DmaC[ DMA_CH2 ].memAdd = Ps2Bios.DmaC[ DMA_CH2 ].tag + DMA_TRS_TAG_SIZE ;
	        	
				DMA_GS_Transfer_Normal_Mode();	
    	        	
	        	/*Compute the Adress after the DATA*/
	        	Ps2Bios.DmaC[ DMA_CH2 ].tag = Ps2Bios.DmaC[ DMA_CH2 ].memAdd  + (  ( Ps2Bios.DmaC[ DMA_CH2 ].size  )  * DMA_TRS_TAG_SIZE );
	        	
	        	break;
	        }
	        
/*	              
	        case DMA_SRC_CHAIN_MODE_NEXT : {
	        	if ( debug_dma == 1 ) 	PrintToDmaOutput (  "CHAINED TRANSFERT TAG : next NI\n" );
	        		        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }
*/        
	        case DMA_SRC_CHAIN_MODE_REF : {
	        	if ( debug_dma == 1 )	PrintToDmaOutput (  "GS:Trans.Chain.:REF\n" );

			
				DMA_GS_Transfer_Normal_Mode();


	        	/*Compute the Adress after the DATA*/
	        	Ps2Bios.DmaC[ DMA_CH2 ].tag = Ps2Bios.DmaC[ DMA_CH2 ].tag  +  DMA_TRS_TAG_SIZE;
       		        	
	        
	        	break;
	        }
/*	        
	        case DMA_SRC_CHAIN_MODE_REFS : {
	        	if ( debug_dma == 1 ) PrintToDmaOutput (  "CHAINED TRANSFERT TAG : refs NI\n" );
	        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }
	        
	        case DMA_SRC_CHAIN_MODE_CALL : {
	        	if ( debug_dma == 1 ) 	PrintToDmaOutput (  "CHAINED TRANSFERT TAG : call NI\n" );
	        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }
	        
	        case DMA_SRC_CHAIN_MODE_RET : {
	        	if ( debug_dma == 1 ) 	PrintToDmaOutput ( "CHAINED TRANSFERT TAG : ret NI\n");
	        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }
*/	        
	        case DMA_SRC_CHAIN_MODE_END : {
	        	if ( debug_dma == 1 ) 	PrintToDmaOutput (  "GS:Trans.Chain.:END\n" );
	        	        	
	        	/*Compute Address after the Tag*/
	        	Ps2Bios.DmaC[ DMA_CH2 ].memAdd = Ps2Bios.DmaC[ DMA_CH2 ].tag + DMA_TRS_TAG_SIZE ;


	        	
				DMA_GS_Transfer_Normal_Mode();	
  	        	
	        	                        		
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        }
	        
	        default : {
	        	/*if ( debug_dma == 1 )*/ 	PrintToEmuOutput (  "DMA GIF CHAINED TRANSFERT TAG : unknown\n" );
	        	
	        	DChainTransfertCompleted = TRUE;
	        	break;
	        		
	        }
	        
	        
	    }//End Switch DMA Chained TAG
	
	}//End While Dma Chained Transfer
}


void DMA_GS_Transfer ( void ) {

	char 	DmaTempText[256];

	/* if the control value is START */
	if ( ( Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0x100 ) == 0x100 ) {
	
		/*Normal Transfert or Chained Transfer*/
		switch ( Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0xFF ) {
		
			/*Normal Transfert*/
			case 0x01 : {
				if ( debug_dma == 1 ) {
					sprintf(DmaTempText,"**Mode Normal: Add:%8.8llx size:%d\n", (long)Ps2Bios.DmaC[DMA_CH2].memAdd, ( long ) Ps2Bios.DmaC[DMA_CH2].size);
					PrintToDmaOutput (  DmaTempText );	
				}
				
				DMA_GS_Transfer_Normal_Mode( );
				nb_dma_transfert++;
				
				break;
			}
			
			/*Chained Transfert - Incomplete*/
			case 0x05 : {
				if ( debug_dma == 1 ) {
					sprintf(DmaTempText,"**Mode Chain: Add:%8.8llx \n", Ps2Bios.DmaC[DMA_CH2].tag);
					PrintToDmaOutput (  DmaTempText );
				}
				
				DMA_GS_Transfer_SourceChain_Mode( )	;
				nb_dma_transfert++;
				
				break;
			
			}
			
			
			/*Other Transfer Mode to GS Not Implemented*/
			default: 	if ( debug_dma == 1 ) {
				sprintf(DmaTempText,"**Mode Not Handled : %2.2llx \n", (Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0xFF )); break;
				PrintToDmaOutput (  DmaTempText );
			}


		
											
		}
	
		if ( debug_dma == 1 ) PrintToDmaOutput (  " \n" );


		/*This call write the end of the dma transfert
		with writing 0 to the bit start of the DM2 channel Control */
		Ps2Bios.DmaC[ DMA_CH2 ].ctrl = Ps2Bios.DmaC[ DMA_CH2 ].ctrl &  0xFFFFFEFF ;
		MipsMemoryWriteW ( SpecialAreaMemory,  Ps2Bios.DmaC[ DMA_CH2 ].ctrl , 0xa000 );	/* don't need DMA check */	
	
	}
				
}









