/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include "nSX2.h"
#include "PS2Edefs.h"


void DMA_SetChannelInterruptStatus( _u8 channelNumber ) {
	_u128 dmac_stat;
    _u32  mask;
	

	mask = 0x1 << channelNumber ;

	//Read the Dmac channel Interrupt Status register
	MipsMemoryRead ( &dmac_stat , DATA_W ,  DMA_STAT  ,0 );

	
    dmac_stat.lo = dmac_stat.lo | mask ;
    
    
    MipsMemoryWrite ( dmac_stat  , DATA_W ,  DMA_STAT  ,0 ) ;
	
	PrintToDmaOutput (  "Enable DMA TRANSFERT WITH INTERRUPT\n" );
}

void DMA_PrintAccesInfo() {
   char DmaTempText[256];
		sprintf(DmaTempText,"%8.8llx:WR:0x1000%4.4llx: ", (long)MipsCpu.PC, Ps2Bios.DmaAreaWriteAt);
		PrintToDmaOutput (  DmaTempText );


}


void		DmaUpdate() {
	_u32 	boucle;
	_u64    Mask;
	_u128	dmac_stat;
	
   char DmaTempText[256];
	
	
/* if a memory write acces to the DMA Register is detected */
if ( Ps2Bios.WriteToDmaArea == 1)  {
	

	switch ( Ps2Bios.DmaAreaWriteAt ) {

/********************************************************************************************/
/*			DMA CH0 REGISTERS (Linked to VIF0)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
  		/*Channel 0  - VIF0 - CHCRT*/
    	case 0x8000: {
			if ( debug_dma == 1 ) {
   				DMA_PrintAccesInfo();
       			sprintf(DmaTempText,"DMAC0_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH0 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH0 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH0 );
			
			break;
		}

		/*Channel 0  - VIF0 - MADR*/
		case 0x8010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC0_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH0 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 0  - VIF0 - SIZE*/
		case 0x8020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC0_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH0 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 0  - VIF0 - TAG*/
		case 0x8030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC0_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH0 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 0  - VIF0 - ASR0*/
		case 0x8040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC0_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH0 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 0  - VIF0 - ASR1*/
		case 0x8050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC0_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH0 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		/*Channel 0  - VIF0 - SADR*/
		case 0x8080: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC0_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH0 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		

/********************************************************************************************/
/*			DMA CH1 REGISTERS (Linked to VIF1)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
  		/*Channel 1  - VIF1 - CHCRT*/
    	case 0x9000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC1_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH1 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH1 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH1 );
			
			break;
		}

		/*Channel 1  - VIF1 - MADR*/
		case 0x9010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC1_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH1 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 1  - VIF1 - SIZE*/
		case 0x9020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC1_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH1 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 1  - VIF1 - TAG*/
		case 0x9030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC1_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH1 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 1  - VIF1 - ASR0*/
		case 0x9040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC1_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH1 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 1  - VIF1 - ASR1*/
		case 0x9050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC1_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH1 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		/*Channel 1  - VIF1 - SADR*/
		case 0x9080: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC1_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH1 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}



/********************************************************************************************/
/*			DMA CH2 REGISTERS (Linked to GIF)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
		
		/*Channel 2  - GIF - CHCRT*/
		case 0xA000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC2_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH2 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH2 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH2 );
			
			DMA_GS_Transfer ();
					
			break;
		}
		
		/*Channel 2  - GIF - MADR*/
		case 0xA010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC2_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH2 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 2  - GIF - SIZE*/
		case 0xA020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC2_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH2 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 2  - GIF - TAG*/
		case 0xA030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC2_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH2 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 2  - GIF - TAG*/
		case 0xA040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC2_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH2 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 2  - GIF - ASR1*/
		case 0xA050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC2_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH2 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 2  - GIF - SADR*/
		case 0xA080: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC2_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
   			}
   				
			Ps2Bios.DmaC[ DMA_CH2 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}

/********************************************************************************************/
/*			DMA CH3 REGISTERS (Linked to IPU (FROM IPU))	- NEED IMPLEMENTATION			*/
/********************************************************************************************/		
  		/*Channel 3  - FROM IPU - CHCRT*/
    	case 0xB000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC3_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH3 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH3 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH3 );
			
			break;
		}

		/*Channel 3  - FROM IPU - MADR*/
		case 0xB010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC3_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH3 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 3  - FROM IPU - SIZE*/
		case 0xB020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC3_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH3 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		/*Channel 3  - FROM IPU - TAG*/
		case 0xB030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC3_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH3 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 3  - FROM IPU - TAG*/
		case 0xB040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC3_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH3 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 3  - FROM IPU - ASR1*/
		case 0xB050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC3_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH3 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 3  - FROM IPU - SADR*/
		case 0xB080: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC3_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
   			}
   				
			Ps2Bios.DmaC[ DMA_CH3 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		



/********************************************************************************************/
/*			DMA CH4 REGISTERS (Linked to IPU (TO IPU))	- NEED IMPLEMENTATION			*/
/********************************************************************************************/		
  		/*Channel 4  - TO IPU - CHCRT*/
    	case 0xB400: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC4_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH4 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH4 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH4 );
			
			break;
		}

		/*Channel 4  - TO IPU - MADR*/
		case 0xB410: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC4_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH4 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 4  - TO IPU - SIZE*/
		case 0xB420: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC4_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH4 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 4  - TO IPU - TAG*/
		case 0xB430: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC4_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH4 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}

	
		/*Channel 4  - TO IPU - ASR0*/
		case 0xB440: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC4_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH4 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 4  - TO IPU - ASR1*/
		case 0xB450: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC3_ASR4 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH4 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 4  - TO IPU - SADR*/
		case 0xB480: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC4_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
   			}
   				
			Ps2Bios.DmaC[ DMA_CH4 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		
		
		
/********************************************************************************************/
/*			DMA CH5 REGISTERS (Linked to SIF0)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
		case 0xC000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	   			sprintf(DmaTempText,"DMAC5_CHCR = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH5 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
        	if ( (Ps2Bios.DmaC[ DMA_CH5 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH5 );
			
			break;
		}
		
		
		case 0xC010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	   			sprintf(DmaTempText,"DMAC5_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
	      		PrintToDmaOutput (  DmaTempText );
	   		}
   			
			Ps2Bios.DmaC[ DMA_CH5 ].memAdd = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			break;
		}
		
		
		
		case 0xC020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC5_QWC = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaC[ DMA_CH5 ].size = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			break;
		}


/********************************************************************************************/
/*			DMA CH6 REGISTERS (Linked to SIF1)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
  		/*Channel 6  - to SIF1 - CHCRT*/
    	case 0xC400: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC6_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH6 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH6 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH6 );
			
			break;
		}

		/*Channel 6  - to SIF1 - MADR*/
		case 0xC410: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC6_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH6 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 6  - to SIF1 - SIZE*/
		case 0xC420: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC6_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH6 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 6  - to SIF1 - TAG*/
		case 0xC430: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC6_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH6 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		

/********************************************************************************************/
/*			DMA CH7 REGISTERS (Linked to SIF2)	- NEED IMPLEMENTATION						*/
/********************************************************************************************/		
  		/*Channel 7  - to SIF2 - CHCRT*/
    	case 0xC800: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC7_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH7 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH7 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH7 );
			
			break;
		}

		/*Channel 7  - to SIF2 - MADR*/
		case 0xC810: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC7_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH7 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 7  - to SIF2 - SIZE*/
		case 0xC820: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC7_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH7 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
/********************************************************************************************/
/*			DMA CH8 REGISTERS (Linked to SPR  (from SCRATCH PAD)	- NEED IMPLEMENTATION	*/
/********************************************************************************************/		
  		/*Channel 8  - from SCRATCH PAD - CHCRT*/
    	case 0xD000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC8_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH8 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH8 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH8 );
			
			break;
		}

		/*Channel 8  - from SCRATCH PAD- MADR*/
		case 0xD010: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC8_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH8 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 8  - from SCRATCH PAD- SIZE*/
		case 0xD020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC8_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH8 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		/*Channel 8  - from SCRATCH PAD - TAG*/
		case 0xD030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC8_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH8 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}

	
		/*Channel 8  - from SCRATCH PAD - ASR0*/
		case 0xD040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC8_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH8 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 8  - from SCRATCH PAD - ASR1*/
		case 0xD050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC8_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH8 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}




		/*Channel 8  - from SCRATCH PAD- SADR*/
		case 0xD080: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC8_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
   			}
   				
			Ps2Bios.DmaC[ DMA_CH8 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}

/********************************************************************************************/
/*			DMA CH9 REGISTERS (Linked to SPR  (To SCRATCH PAD)	- NEED IMPLEMENTATION		*/
/********************************************************************************************/		
  		/*Channel 9  - To SCRATCH PAD - CHCRT*/
    	case 0xD400: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC9_CHCRT = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
  			}
			
			Ps2Bios.DmaC[ DMA_CH9 ].ctrl = Ps2Bios.DmaAreaWriteVal;		//store the control value
			
			if ( (Ps2Bios.DmaC[ DMA_CH9 ].ctrl & 0x80 ) == 0x80 ) DMA_SetChannelInterruptStatus( DMA_CH9 );
			
			break;
		}

		/*Channel 9  - To SCRATCH PAD - MADR*/
		case 0xD410: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC9_MADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH9 ].memAdd = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 9  - To SCRATCH PAD - SIZE*/
		case 0xD420: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC9_SIZE = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
			}
   				
			Ps2Bios.DmaC[ DMA_CH9 ].size = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
  		/*Channel 9  - To SCRATCH PAD - TAG*/
		case 0xD430: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
				sprintf(DmaTempText,"DMAC9_TAG = %llx\n",Ps2Bios.DmaAreaWriteVal);
				PrintToDmaOutput (  DmaTempText );
   			}
   			
			Ps2Bios.DmaC[ DMA_CH9 ].tag = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		/*Channel 9  - To SCRATCH PAD - ASR0*/
		case 0xD440: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC9_ASR0 = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
 			}
   				
			Ps2Bios.DmaC[ DMA_CH9 ].asr0 = Ps2Bios.DmaAreaWriteVal;
			break;
		}
		
		/*Channel 9  - To SCRATCH PAD - ASR1*/
		case 0xD450: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	  			sprintf(DmaTempText,"DMAC9_ASR1 = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
   			
			Ps2Bios.DmaC[ DMA_CH9 ].asr1 = Ps2Bios.DmaAreaWriteVal;
			break;
		}


		/*Channel 9  - To SCRATCH PAD - SADR*/
		case 0xD480: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
   				sprintf(DmaTempText,"DMAC9_SADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
   				PrintToDmaOutput (  DmaTempText );
   			}
   				
			Ps2Bios.DmaC[ DMA_CH9 ].saddr = Ps2Bios.DmaAreaWriteVal;
			break;
		}

		
		
/********************************************************************************************/
/*			DMA CONTROL REGISTERS	- NEED IMPLEMENTATION									*/
/********************************************************************************************/		
		case 0xE000: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_CTRL = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.ctrl = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}		
  
  
  		/* Interrupt Status register - NEED IMPLEMENTATION*/
  		/*
  		bit0-bit9: 	Channel Interrupt Status. Set to 1 when transfer processing via each channel ends.
  					Cleared when 1 is written.
  		bit13:		DMA Stall interrupt status.Set to 1 when stall condition is met.
  					Cleared when 1 is written.
       	bit14:
       	bit15:
       	bit16-bit25: Channel Interrupt Mask. 0 Disable. 1 Enable - reverssed when 1 is written.
  		
  		Can't be done here but in r5900_memory.c
  		
  		*/
  		case 0xE010: {
			
   			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
	   			sprintf(DmaTempText,"DMAC_C_STAT  = %llx\n",Ps2Bios.DmaAreaWriteVal);
	   			PrintToDmaOutput (  DmaTempText );
	   			}
			
			Ps2Bios.DmaControler.stat = Ps2Bios.DmaAreaWriteVal;
 			
			break;
		}
		
		case 0xE020: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_PCR = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.pcr = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}		
		
		case 0xE030: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_SQWC = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.sqwc = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}
		
  		case 0xE040: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_RBSR = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.rbsr = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}
  
		case 0xE050: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_RBOR = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.rbor = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}    

		case 0xE060: {
			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"DMAC_C_STADR = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   		
			Ps2Bios.DmaControler.stadr = Ps2Bios.DmaAreaWriteVal;
			
			break;
		}      
 /********************************************************************************************/
/*			Interrup by Cause  REGISTERS	- NEED IMPLEMENTATION									*/
/********************************************************************************************/		
		
   		case 0xF000: {
			
   			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"INTC_STAT = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   			
			
			
			break;
		}           


   		case 0xF010: {
			
   			if ( debug_dma == 1 ) { DMA_PrintAccesInfo();
		   		sprintf(DmaTempText,"INTC_MASK = %llx\n",Ps2Bios.DmaAreaWriteVal);
		   		PrintToDmaOutput (  DmaTempText );
   			}
   			
			
			
			break;
		}


          		
		default: 	{
  			sprintf(DmaTempText,"DMA Adress Non-Impl.  0x1000%4.4llx", Ps2Bios.DmaAreaWriteAt);
  			PrintToEmuOutput (  DmaTempText );
  			}
				
				
	
	}
	
	
	
	
	/* the DMA Register access is DONE */
	Ps2Bios.WriteToDmaArea = 0;
	}


	/* if a memory read acces to the DMA Register is detected */
	if ( Ps2Bios.ReadToDmaArea == 1)  {
	
		switch ( Ps2Bios.DmaAreaReadAt ) {
   		
		default:
	
			if ( debug_dma == 1 )  {
  		    			sprintf(DmaTempText,"%8.8llx:RD:0x1000%4.4llx: 0x%8.8llx\n", MipsCpu.PC, Ps2Bios.DmaAreaReadAt,Ps2Bios.DmaAreaReadVal);
    			PrintToDmaOutput (  DmaTempText );
	    	}
    	}
		
		Ps2Bios.ReadToDmaArea = 0;
	}
		
}


/*
This function need to be converted to the GTK GUI
*/
void DmaPrintChannelInfo ( _u8 fChannel ) {

	printf("\nDMA Channel n %2.2d\n",fChannel);
	printf("Channel Control register: %llx\n",Ps2Bios.DmaC[fChannel].ctrl);
	printf("Memory address          : %llx\n",Ps2Bios.DmaC[fChannel].memAdd);
	printf("Quadra-word counter     : %llx\n",Ps2Bios.DmaC[fChannel].size);
	printf("Tag address             : %llx\n",Ps2Bios.DmaC[fChannel].tag);
	printf("Assembler Reserved 0    : %llx\n",Ps2Bios.DmaC[fChannel].asr0);
	printf("Assembler Reserved 1    : %llx\n",Ps2Bios.DmaC[fChannel].asr1);
	printf("???                     : %llx\n",Ps2Bios.DmaC[fChannel].saddr);
	
	
}

