/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>

#include "nSX2.h"

/*
Execute an Handler at the adress specified by HandlerAdress
NOTA: __inline means compiler do not optimize the funstion
*/
static __inline void BIOS_ExecHandler(_u32 HandlerAdress) {
	_u64 SavedRA, SavedPC;

	/* Save the Program Counter and the Return Adress */
	SavedPC = MipsCpu.PC;
	SavedRA = MipsCpu.GPR[CPU_REG_RA].lo;
	
	/* Change the PC and RA */
	MipsCpu.PC = HandlerAdress;
	MipsCpu.GPR[CPU_REG_RA].lo = 0x80001000;	//This values is inside KSEG0 ( Kernel Segment 0 )

	/*Execute the Handler */
 	while ((_u32)MipsCpu.PC != (_u32)0x80001000) {
		MipsUpdateCpu ( );
		
		// Prevents some errors on exec Handler 
		if ( MipsCpu.halt != 0 ) MipsCpu.PC = 0x80001000;
	}

	/* Restore Initial PC and RA values */
 	MipsCpu.GPR[CPU_REG_RA].lo = SavedRA;
	MipsCpu.PC = SavedPC;
}



void BiosUpdate( void ) {
	char temp_string[256];		//Debug text String
	_u8 boucle;					//loop var
	_u128	intc_stat;			//INTC_STAT value ( at adress 0x1000F000 )
	_u128	dma_stat;			//DMAC_STAT value ( at adress 0x1000E010 )

	_u128 		GPR_temp[34];	//BackUp of GeneralRegisters
	
	/* Check the Interrupt by Cause Handler */
	for ( boucle=0; boucle < BIOS_MAX_INTC_HANDLER; boucle++ ) { 
	
		/* Read INTC_STAT value*/
		MipsMemoryReadW ( SpecialAreaMemory, &intc_stat , (INTC_STAT & 0xFFFF) ) ;	//don't need DMA Check : MipsMemoryRead ( &intc_stat , DATA_W ,  INTC_STAT  ,0 );

		/* If an interrupt occurs on the  specified Cause ( boucle value )*/
		if (intc_stat.lo & ( 1<< boucle ) ) {

			/* If the Intc Handler is Active */
			if ( Ps2Bios.IntCH[ boucle ].Status ) {
				/* Print Debug Info */
				if ( debug_print_syscall == 1 )	{ 
	   				sprintf(temp_string,"--Exec IntrptC:cause %d ( %s )\n",boucle,debugIntrptCause[boucle]);PrintToBIOSOutput (  temp_string );
     		    }
	   		
	   		    /* Save all the General Register */
	   			memcpy(&GPR_temp, &(MipsCpu.GPR), (sizeof(struct __u128))*34);
	   		
	   			/* Put Cause in A0 */	
				MipsCpu.GPR[CPU_REG_A0].lo = boucle;		
			
				/* Exec the Handler */
				BIOS_ExecHandler( Ps2Bios.IntCH[ boucle ].Add );
				
				/*Restore the General Register */
				memcpy(&(MipsCpu.GPR), &GPR_temp, (sizeof(struct __u128))*34);

				/* Set to 0 the flag of the interrupt */	
				intc_stat.lo = intc_stat.lo & ~(1<<boucle);
				MipsMemoryWriteW ( SpecialAreaMemory, intc_stat.lo , (INTC_STAT & 0xFFFF) ) ;	//don't need DMA Check : MipsMemoryRead ( &intc_stat , DATA_W ,  INTC_STAT  ,0 );
				
				//if ( debug_print_syscall == 1 )	 PrintToBIOSOutput (  "--End IntrptC Handler\n" );			


			} /* end of Handler Active */
		
		} /* end of Interrupt occur */

	} /* end of for */


		
	/* Check the DMA Interrupt Handler */
	for (boucle=0; boucle<BIOS_MAX_DMA_HANDLER; boucle++) { 

		/* Read DMAC_STAT value*/	
		MipsMemoryReadW ( SpecialAreaMemory, &dma_stat , (DMA_STAT & 0xFFFF) ) ;	//don't need DMA Check
		
		/* If an interrupt occurs on the  specified Channel ( boucle value )*/	
		if (dma_stat.lo & (1<<boucle)) {
			
			/* If the Dma Handler is Active */
			if (Ps2Bios.DmaH[ boucle ].Status ) {
					
				/* Print Debug Info */
				if ( debug_print_syscall == 1 )	{ 
     				sprintf(temp_string,"--Exec DMA Handler Channel: %d\n",boucle);PrintToBIOSOutput (  temp_string );
       				}

				/* Save all the General Register */
   				memcpy(&GPR_temp, &MipsCpu.GPR, sizeof(_u128)*34);
			
				/* Put Channel number in A0 */
				MipsCpu.GPR[CPU_REG_A0].lo = boucle;

				/* Exec the Handler */		
				BIOS_ExecHandler(Ps2Bios.DmaH[ boucle ].Add);
   				
				/*Restore the General Register */
				memcpy(&MipsCpu.GPR, &GPR_temp, sizeof(_u128)*34);

				/* Set to 0 the flag of the interrupt */
				dma_stat.lo = dma_stat.lo & ~(1<<boucle);
				MipsMemoryWriteW ( SpecialAreaMemory, dma_stat.lo , (DMA_STAT & 0xFFFF) ) ;	//don't need DMA Check : MipsMemoryRead ( &intc_stat , DATA_W ,  INTC_STAT  ,0 );

				//if ( debug_print_syscall == 1 )	 PrintToBIOSOutput (  "--End DMA Handler\n" );
      			
			}/* end of Handler Active */
		
		}/* end of Interrupt occur */
		
	}/* end of for */





}

