/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdlib.h>
#include <stdio.h>

#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "nSX2.h"
#include "elf_file.h"

FILE * ElfFile;
elf_header ico_elf_hdr;
elf_program_header ico_elf_prg_hdr[30];
elf_section_header ico_elf_sec_hdr[30];
long ElfFileSize;

FILE * BiosFile;


long FileSizeOf( FILE *fp )
{
    long int save_pos;
    long size_of_file;

    save_pos = ftell( fp );
    fseek( fp, 0L, SEEK_END );
    size_of_file = ftell( fp );
    fseek( fp, save_pos, SEEK_SET );
    return( size_of_file );
}


_u8 ElfOpenFile( char * fFileName ) {

	_u8 	Result;
	char tempText[100];
//	 int errvalue;
	
	ElfFile = fopen( fFileName, "rb" );
	if( ElfFile != NULL )  {
		ElfFileSize = FileSizeOf( ElfFile ) ;
		
		sprintf(tempText,"File:Opening Elf : %s ( size : %5.3f Ko)" ,fFileName , ( (float) ElfFileSize  / 1024) );	
		PrintToEmuOutput(tempText);	
		
		Result = TRUE; 
	}
	else	{
		sprintf(tempText,"File:Error opening :%s: %s", fFileName ,strerror( errno ) );
		PrintToEmuOutput(tempText);					
		
		Result = FALSE;
	}
	
	return Result;	
}

_u8 BiosOpenFile( char * fFileName ) {

	_u8 	Result;
	long BiosFileSize;
	char tempText[100];
	
	BiosFile = fopen( fFileName, "rb" );
	if( BiosFile != NULL )  {
		BiosFileSize = FileSizeOf( BiosFile ) ;
		
		sprintf(tempText,"File:Opening BIOS : %s ( size : %5.3f Ko)" ,fFileName , ((float) BiosFileSize  / 1024) );
		PrintToEmuOutput(tempText);			
		
		Result = TRUE; 
	}
	else	{
		sprintf(tempText,"File:Error opening %s", fFileName );
		PrintToEmuOutput(tempText);	
		Result = FALSE;
	}
	
	return Result;	
}


void ElfCloseFile( void ) {
		char tempText[100];
		
		memset(	tempText , 0 , 100 );	
		sprintf(tempText,"File:Close Elf File");
		PrintToEmuOutput(tempText);		
		
		
		if ( ElfFile != NULL ) fclose( ElfFile );
}

void BiosCloseFile( void ) {
		char tempText[100];
		
		
		memset(	tempText , 0 , 100 );			
		sprintf(tempText,"File:Close Bios File");
		PrintToEmuOutput(tempText);
		
			
		if ( BiosFile != NULL ) fclose( BiosFile ); 
}

_u8 ElfLoadHeader() {
	
	_u8  Index;
	_u8 Result;
	char tempText[100];
		
	Result = 0;
	
	fseek( ElfFile, 0L, SEEK_SET );
	if ( fread( &ico_elf_hdr , sizeof(struct  _elf_header ), 1, ElfFile ) != 0 ) {
		sprintf(tempText,"File:Read Elf Header ");
		PrintToEmuOutput(tempText);			
		MipsCpu.EntryPointAdd = ico_elf_hdr.entry_point_add;
		}
	else Result++;
	
	fseek( ElfFile, ico_elf_hdr.start_prg_hdr, SEEK_SET );
	
	for ( Index = 0 ; Index <  ico_elf_hdr.num_of_prg_hdr ; Index++ ) {
		
		if ( fread( &ico_elf_prg_hdr[Index] , sizeof(struct  _elf_program_header ), 1, ElfFile ) == 0 ) Result++;
		
		//fseek( ElfFile, ico_elf_hdr.start_prg_hdr+ ( sizeof(struct  _elf_program_header ) * (Index + 1) ) +ico_elf_prg_hdr[Index].num_of_bytes_in_file_ima , SEEK_SET );
	}
	
	sprintf(tempText,"File:Read Elf Program Header Numero 0 to %d",Index);
	PrintToEmuOutput(tempText);		
	
	fseek( ElfFile,  ico_elf_hdr.start_sct_hdr, SEEK_SET );

	for ( Index = 0 ; Index <  ico_elf_hdr.num_of_sec_hdr ; Index++ ) {
		if ( fread( &ico_elf_sec_hdr[Index] , sizeof(struct  _elf_section_header ), 1, ElfFile ) == 0 )	Result++;
			
	}	
	
	sprintf(tempText,"File:Read Elf Section Header Numero 0 to %d",Index);
	PrintToEmuOutput(tempText);
	
	if ( Result == 0 ) 	sprintf(tempText,"File:Read Elf Header DONE");
	else sprintf(tempText,"File:Read Elf Header ERROR");
	PrintToEmuOutput(tempText);
	

	return Result;
}

/*
Debug Info
*/
void ElfPrintInfo() {
	_u8	index;
	char tempText[100];	

	sprintf(tempText,"--------------------------------------------------------------------------");PrintToEmuOutput(tempText);
	sprintf(tempText,"ELF INFO");PrintToEmuOutput(tempText);
	sprintf(tempText,"Entry Point Adresse : %lx", ico_elf_hdr.entry_point_add );PrintToEmuOutput(tempText);
	sprintf(tempText,"\n");PrintToEmuOutput(tempText);	
	sprintf(tempText,"Start Of Program Header : %lx", ico_elf_hdr.start_prg_hdr );PrintToEmuOutput(tempText);
	sprintf(tempText,"Number Of  Program Header : %x", ico_elf_hdr.num_of_prg_hdr );PrintToEmuOutput(tempText);
	for( index = 0 ; index < ico_elf_hdr.num_of_prg_hdr; index++) {
		sprintf(tempText,"ProgramHeader %d",index);PrintToEmuOutput(tempText);
		sprintf(tempText," Offset from file start to program segment : %lx", ico_elf_prg_hdr[index].offs_from_start_to_prg_seg );PrintToEmuOutput(tempText);
		sprintf(tempText," Virtual Adress of Segement : %lx", ico_elf_prg_hdr[index].virtual_add_of_segment );PrintToEmuOutput(tempText);
		sprintf(tempText," Physical Adress of Segment  : %lx", ico_elf_prg_hdr[index].physic_add_of_segment );PrintToEmuOutput(tempText);
		sprintf(tempText," Number of bytes in file image  : %lx ( %ld ) ", ico_elf_prg_hdr[index].num_of_bytes_in_file_ima, ico_elf_prg_hdr[index].num_of_bytes_in_file_ima );PrintToEmuOutput(tempText);
		sprintf(tempText," Number of bytes in mem image : %lx ( %ld ) ", ico_elf_prg_hdr[index].num_of_bytes_in_mem_ima, ico_elf_prg_hdr[index].num_of_bytes_in_mem_ima );PrintToEmuOutput(tempText);
	}
	sprintf(tempText,"\n");PrintToEmuOutput(tempText);
	sprintf(tempText,"Start Of Section Header : %lx ( %ld )", ico_elf_hdr.start_sct_hdr, ico_elf_hdr.start_sct_hdr  );PrintToEmuOutput(tempText);
	sprintf(tempText,"Number of  Section Header : %x ( %d )", ico_elf_hdr.num_of_sec_hdr ,  ico_elf_hdr.num_of_sec_hdr );PrintToEmuOutput(tempText);
	sprintf(tempText,"Section Header Entry Size: %x ( %d )", ico_elf_hdr.sec_hdr_entry_size,  ico_elf_hdr.sec_hdr_entry_size);PrintToEmuOutput(tempText);
	sprintf(tempText,"Section Header Stringatable Index: %x ( %d )\n", ico_elf_hdr.sec_hdr_stringa_index, ico_elf_hdr.sec_hdr_stringa_index );PrintToEmuOutput(tempText);
	
	
	for( index = 0 ; index < ico_elf_hdr.num_of_sec_hdr; index++) {
		ElfPrintSectionInfo( index );
		
	}	
		
	sprintf(tempText,"--------------------------------------------------------------------------");PrintToEmuOutput(tempText);
}



void ElfPrintSectionInfo( _u8 fIndex ) {
	char tempText[100];	

	sprintf(tempText,"--------------------------------------------------------------------------");PrintToEmuOutput(tempText);

	sprintf(tempText,"Section numero : %d",fIndex);PrintToEmuOutput(tempText);
	sprintf(tempText," Type : %lx ( %ld )", ico_elf_sec_hdr[fIndex].type, ico_elf_sec_hdr[fIndex].type );PrintToEmuOutput(tempText);
	sprintf(tempText," Flag : %lx ( %ld )", ico_elf_sec_hdr[fIndex].sect_flag, ico_elf_sec_hdr[fIndex].sect_flag );PrintToEmuOutput(tempText);
	sprintf(tempText," Numero Index Section : %lx ( %ld )", ico_elf_sec_hdr[fIndex].numero_index_sect,  ico_elf_sec_hdr[fIndex].numero_index_sect );PrintToEmuOutput(tempText);
	sprintf(tempText," Section Start Adress : %lx ( %ld )", ico_elf_sec_hdr[fIndex].sect_start_add, ico_elf_sec_hdr[fIndex].sect_start_add );PrintToEmuOutput(tempText);
	sprintf(tempText," Offset from start : %lx ( %ld )", ico_elf_sec_hdr[fIndex].offset_from_start ,  ico_elf_sec_hdr[fIndex].offset_from_start  );PrintToEmuOutput(tempText);
	sprintf(tempText," Size : %lx ( %ld ) ", ico_elf_sec_hdr[fIndex].size,  ico_elf_sec_hdr[fIndex].size  );PrintToEmuOutput(tempText);
	sprintf(tempText," Alignement : %lx ( %ld ) ", ico_elf_sec_hdr[fIndex].alignement,  ico_elf_sec_hdr[fIndex].alignement  );PrintToEmuOutput(tempText);
	sprintf(tempText," Fixed Size Entry : %lx ( %ld ) ", ico_elf_sec_hdr[fIndex].fixed_size_entry, ico_elf_sec_hdr[fIndex].fixed_size_entry);PrintToEmuOutput(tempText);

	sprintf(tempText,"--------------------------------------------------------------------------");PrintToEmuOutput(tempText);
}
