/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <gtk/gtk.h> 

#include <gmodule.h>

#include "nSX2.h"
#include "PS2Edefs.h"

#include "debug_interface.h"


extern char * debug_r5900_reg [34];

/*Widget Variables Declarations*/
GtkWidget *win; 
GtkWidget *vbox;
GtkWidget *menu_bar;
GtkWidget *submenu;
GtkWidget *menu_item;
GtkWidget *scrollbar;

static GtkWidget *StatusBar; 
guint ContextId1;
guint ContextId2;

guint	KeySnooperID;

GtkWidget *EmuOutput;


/*************************************/

/*
Timer to compute the Instruction Per Second
*/
GTimer * GTimer_IPS;


/* GTK Functions Declarations */
guint 	nSX2Idle_funcID;
int 	nSX2InitProc  		(GtkWidget *pWidget, gpointer pData );
int 	nSX2MainEmuProc 	(GtkWidget *pWidget, gpointer pData );
int 	nSX2Idle_nullfunc	(GtkWidget *pWidget, gpointer pData );

void 	arreter_boucle		(GtkWidget* widget);
void 	arreter_debugWindow	(GtkWidget* widget);
void    gtkErrorEmulator    (GtkWidget* widget, gpointer data);

void 	Quitter				(GtkWidget* widget, gpointer data);
void 	About				(GtkWidget* widget, gpointer data);

void 	gtkcb_LoadElf		(GtkWidget* widget, gpointer data);
void 	recuperer_chemin 	(GtkWidget* bouton, GtkWidget *file_selection);

void 	gtkcb_EmuRun		(GtkWidget* widget, gpointer data);
void 	gtkcb_EmuPause		(GtkWidget* widget, gpointer data);
void 	gtkcb_DbgShow 		(GtkWidget* widget, gpointer data);

void    gtkcb_CPUConf       (GtkWidget* widget, gpointer data);
void    gtkcb_GSConf       (GtkWidget* widget, gpointer data);
void    gtkcb_InputConf       (GtkWidget* widget, gpointer data);

guint    gtk_nSX2KeySnoop	(GtkWidget *grab_widget,    GdkEventKey *event,  gpointer func_data);

/* nSX2 Variables Declarations */
Mips_CPU 		MipsCpu;
Bios_Hw  		Ps2Bios;
VPU0			VectorUnit0;	


char 			ElfToExecute[256];
char            GSPlugginToUse[256];
char 			InputPlugginToUse[256];
char 			SifPlugginToUse[256];


/* nSX2 Functions Declarations */
float 	ComputeInstructionsPerSec		( void  );

void 	PrintToEmuOutput 				( char * ftext );
void 	PrintToOpcodeOutput 			( char * ftext );
void 	PrintToRegistersOutput 			( char * ftext );
void 	PrintToCOP1RegistersOutput 		( char * ftext );





/*************************************************************************/
/*							GTK:MAIN WINDOW							 	 */
/*************************************************************************/

int main(int argc,char* argv[])
{ 


	char tempText[100];
	_u8		Result = 0;

	_u32	LibInfoVersion;
	_u8		LibVersion;
	_u8		LibRevision;	
	_u8		LibBuild;

	/* Read config File
		If errors happend, no problem this functions load default config
	*/
	nSX2LoadConfig();							

	/*Main GTK Init*/
	gtk_init(&argc,&argv);
 	
	/*Main Window*/
	win = gtk_window_new(GTK_WINDOW_TOPLEVEL);                      //
	sprintf(tempText,"nSX2 %1.1d.%2.2d.%1.1d",  nSX2_VERSION_MAJ, nSX2_VERSION_MIN,nSX2_VERSION_BUILD);
			
    gtk_window_set_title(GTK_WINDOW(win),tempText);            		//Title
	gtk_window_set_default_size(GTK_WINDOW(win),500,400);           //Size
	gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_CENTER );  //Center Position
   	
	
	
	/*Main Box*/
	vbox=gtk_vbox_new(FALSE,0);
	gtk_container_add(GTK_CONTAINER(win),vbox);
	
	/*Main Menu*/
	menu_bar = gtk_menu_bar_new();
	
	/*File Sub Menu*/
	submenu = gtk_menu_new();
	/*Menu Item : Load elf*/
	menu_item = gtk_menu_item_new_with_label("Load Elf");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_LoadElf), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/*Menu Item : Quit*/
	menu_item = gtk_menu_item_new_with_label("Exit");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(Quitter), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/*Sub Menu Name*/
	menu_item = gtk_menu_item_new_with_label("File");
	/*Add Sub Menu to Menu Bar*/
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);
	
	
	/* Emulation Sub Menu*/
	submenu = gtk_menu_new();
	/* */
	menu_item = gtk_menu_item_new_with_label("Run");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_EmuRun), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/* */
	menu_item = gtk_menu_item_new_with_label("Pause");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_EmuPause), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/* */
	menu_item = gtk_menu_item_new_with_label("Emulation");
	/* */
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);
	
	
	/* Debug Sub Menu*/
	submenu = gtk_menu_new();
	/* */
	menu_item = gtk_menu_item_new_with_label("Show");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_DbgShow), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/* */
	menu_item = gtk_menu_item_new_with_label("Debug");
	/* */
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);
	
	
	/* Config Sub Menu */
	submenu = gtk_menu_new();
	/* */
//	menu_item = gtk_menu_item_new_with_label("CPU");
//	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_CPUConf), win);
//	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/* */
	menu_item = gtk_menu_item_new_with_label("GS");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_GSConf), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);

	menu_item = gtk_menu_item_new_with_label("Input");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(gtkcb_InputConf), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);

	/* */
	menu_item = gtk_menu_item_new_with_label("Config");
	/* */
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);
	
	
	/* About Sub Menu */
	submenu = gtk_menu_new();
	/* */
	menu_item = gtk_menu_item_new_with_label("About...");
	g_signal_connect(G_OBJECT(menu_item), "activate", G_CALLBACK(About), win);
	gtk_menu_shell_append(GTK_MENU_SHELL(submenu), menu_item);
	/* */
	menu_item = gtk_menu_item_new_with_label("?");
	/* */
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu_bar), menu_item);
	
	
	/*Emu Output*/
	EmuOutput=gtk_text_view_new();
	
	scrollbar = gtk_scrolled_window_new(NULL, NULL);
	
	gtk_container_add(GTK_CONTAINER(scrollbar),EmuOutput);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollbar), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	
	
	
	/*Status Bar*/
	StatusBar = gtk_statusbar_new();
	
	ContextId1 = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "ExitMsg");
	ContextId2 = gtk_statusbar_get_context_id(GTK_STATUSBAR(StatusBar), "AboutMsg");
	
	gtk_statusbar_push (GTK_STATUSBAR (StatusBar), GPOINTER_TO_INT(ContextId1), " ");
	
	
	/* Add Widget to Box*/
	gtk_box_pack_start(GTK_BOX(vbox), menu_bar, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox),EmuOutput,TRUE,TRUE,0);
	gtk_box_pack_start(GTK_BOX(vbox), scrollbar, TRUE, TRUE, 0);
	gtk_box_pack_end(GTK_BOX(vbox),StatusBar,FALSE,FALSE,0);
	
	/* On destroy window, call "arreter_boucle" function */
	g_signal_connect(G_OBJECT(win),"destroy",G_CALLBACK(arreter_boucle),NULL);
	
	
	/*Load GS Pluggin Function */
    Result = LoadGsPluggin(GSPlugginToUse);
    
    if ( Result != 0 ) {
    	PrintToEmuOutput (  "Emu:Plugin GS Error\nClose nSX2 and check nSX2.ini\n" ) ;
   
    }
    else {
		LibInfoVersion = (* PS2EgetLibVersionGS)();

		LibVersion 	= ( LibInfoVersion >> 16 ) 	& 0xFF;
		LibRevision = ( LibInfoVersion >>  8 ) 	& 0xFF;
		LibBuild	=   LibInfoVersion			& 0xFF;
	
		sprintf(tempText,"Emu:Plugin GS : %s %d.%d loaded \n", (*PS2EgetLibNameGS)(),LibRevision,LibBuild );
    	PrintToEmuOutput (  tempText ) ;
		
		(*GSinit)();
	}


	/*Load Input Pluggin Function */
	Result = LoadInputPluggin( InputPlugginToUse );
    if ( Result != 0 ) {
		sprintf(tempText,"Emu:Plugin Input Error \nname:%s\nClose nSX2 and check nSX2.ini\n", InputPlugginToUse);
    	PrintToEmuOutput (  tempText ) ;
   
    }
    else {
	
		LibInfoVersion = (* PS2EgetLibVersionPAD)();

		LibVersion 	= ( LibInfoVersion >> 16 ) 	& 0xFF;
		LibRevision = ( LibInfoVersion >>  8 ) 	& 0xFF;
		LibBuild	=   LibInfoVersion			& 0xFF;
	
		sprintf(tempText,"Emu:Plugin Input : %s %d.%d loaded \n", (*PS2EgetLibNamePAD)(),LibRevision,LibBuild );
    	PrintToEmuOutput (  tempText ) ;


		(*PAD1init)(1);
	}


	Result = LoadSifPluggin( SifPlugginToUse );
   if ( Result != 0 ) {
		sprintf(tempText,"Emu:Plugin Sif Error \nname:%s\nClose nSX2 and check nSX2.ini\n", SifPlugginToUse);
    	PrintToEmuOutput (  tempText ) ;
   
    }
    else {
	
	
		sprintf(tempText,"Emu:Plugin Sif : loaded \n" );
    	PrintToEmuOutput (  tempText ) ;
		Result = BindSifPluggin();
		if ( Result != 0 )  PrintToEmuOutput (  "Emu:Plugin Sif : Internal Bind Errror \n" ); 
		

		
	}
	
	/* */
    gtk_widget_show_all(win);
	
	
	/* */
	gtk_main();
	
	return 0; 
}


/*************************************************************************/
/*							GTK:QUIT WINDOW								 */
/*************************************************************************/
/*
Show a window when user decides to quit nSX2
*/
void Quitter(GtkWidget* widget, gpointer data)
{
	GtkWidget *question;
	
	question = gtk_message_dialog_new (GTK_WINDOW(data),
	GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
	GTK_MESSAGE_QUESTION,
	GTK_BUTTONS_YES_NO,
	"Really want to quit ?");
	
	switch(gtk_dialog_run(GTK_DIALOG(question)))
	{
		case GTK_RESPONSE_YES: {
			arreter_boucle(data);
			break;
		}
		
		case GTK_RESPONSE_NO: {
			gtk_widget_destroy(question);
			break;
		}
		
		default: break;
	}
}


/*************************************************************************/
/*						GTK:ERROR WINDOW								 */
/*************************************************************************/
/*
Show a window when the initialisation has errors
*/
void gtkErrorEmulator(GtkWidget* widget, gpointer data)
{
	GtkWidget *about;
		
	about = gtk_message_dialog_new (GTK_WINDOW(data),
	GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
	"Sorry, Emulator Error",NULL);
	
	gtk_dialog_run(GTK_DIALOG(about));
	
	gtk_widget_destroy(about);
	
	arreter_boucle(data);
}

/*************************************************************************/
/*						GTK:ABOUT WINDOW								 */
/*************************************************************************/
/*
Show the About window
*/
void About(GtkWidget* widget, gpointer data)
{
	GtkWidget *about;
	gchar *site = "http://nsx2.emulation64.com";

	about = gtk_message_dialog_new (GTK_WINDOW(data),
	GTK_DIALOG_MODAL,GTK_MESSAGE_INFO,GTK_BUTTONS_OK,
	"neutrinoSX2 v%1.1d.%2.2d.%1.1d\nAn open source PS2 Emulator\n\nby muad\n%s",nSX2_VERSION_MAJ, nSX2_VERSION_MIN,nSX2_VERSION_BUILD,site);
	
	gtk_dialog_run(GTK_DIALOG(about));
	
	gtk_widget_destroy(about);
}


/*************************************************************************/
/*						GTK:LOAD FILE SELECTION WINDOW					 */
/*************************************************************************/
/*
When user click on "Load Elf" in the menu
*/
void gtkcb_LoadElf(GtkWidget* widget, gpointer data) {

	GtkWidget *selection;
	
	/* Create a file selection window*/
	selection = gtk_file_selection_new( g_locale_to_utf8( "Select a file", -1, NULL, NULL, NULL) );
	gtk_file_selection_set_filename (GTK_FILE_SELECTION(selection),"demos\\*.elf");
	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION(selection)); 
	
	/*Show the window*/
	gtk_widget_show(selection);
	
	/*Make this window Modal */
	gtk_window_set_modal(GTK_WINDOW(selection), TRUE);
	
	/*Create Event for "OK" & "CANCEL" Button*/
	g_signal_connect(G_OBJECT(GTK_FILE_SELECTION(selection)->ok_button), "clicked", G_CALLBACK(recuperer_chemin), selection );
	g_signal_connect_swapped(G_OBJECT(GTK_FILE_SELECTION(selection)->cancel_button), "clicked", G_CALLBACK(gtk_widget_destroy), selection);


}


/*
Get the path & the filename of the Elf
and
RUN THE INIT FUNCTION
*/
void recuperer_chemin (GtkWidget *bouton, GtkWidget *file_selection)
{
	const gchar* chemin;

	/* Get the filename from the selection window*/
	chemin = gtk_file_selection_get_filename(GTK_FILE_SELECTION (file_selection) );
	/*Destroy the selection window*/
	gtk_widget_destroy(file_selection);
	
	/*Needed for emulation*/
	memset( ElfToExecute,0,256);
	strcpy( ElfToExecute,chemin);
	
	/*
		Change the Idle proc of the Gui
		this is very IMPORTANT
		
		I create an idle function, it is called when the gui do nothing
		
		NOTE: I decide to load elf and to start Execution but
		It is possible to change this
		
	*/
	gtk_idle_remove    (nSX2Idle_funcID);
	nSX2Idle_funcID = gtk_idle_add( (GtkFunction)nSX2InitProc ,  NULL );

} 



/*************************************************************************/
/*						GTK:EMULATION MENU								 */
/*************************************************************************/
/*
When user click on "Emulation/Start" Menu
*/
void gtkcb_EmuRun(GtkWidget* widget, gpointer data) {	MipsCpu.halt = 0; }

/*
When user click on "Emulation/Pause" Menu
*/
void gtkcb_EmuPause(GtkWidget* widget, gpointer data) {	MipsCpu.halt = 1; }




/*************************************************************************/
/*						GTK:DEBUG WINDOW								 */
/*************************************************************************/

/*
When user click on "Debug/Show" Menu
*/
void gtkcb_DbgShow (GtkWidget* widget, gpointer data)
{
	GtkWidget *window_Debug;
	
	/*Create the debug window*/
	window_Debug = create_Dbg_Window ();
	
	/*Create an event , calling when destroy the window*/
	g_signal_connect(G_OBJECT(window_Debug),"destroy",G_CALLBACK(arreter_debugWindow),NULL);
	
	/*Show the debug window*/
	gtk_widget_show (window_Debug);
}

/*
When user see the debug window
and decide to close the window
*/
void arreter_debugWindow(GtkWidget* widget) {
	
	debug_op 				= 0;
	debug_gs 				= 0;
	debug_dma 				= 0;
	debug_print_syscall 	= 0;

}

/*
When user see the debug window
and when click on run button
*/
void gtkcb_DebugEmuRun(GtkWidget* widget, gpointer data)
{
	debug_op = 1;
	MipsCpu.halt = 0;
	
}


/*
When user see the debug window
and when click on pause button
*/

void gtkcb_DebugEmuPause(GtkWidget* widget, gpointer data)
{
	debug_op = 0;
	MipsCpu.halt = 1;
	
}

/*
When user see the debug window
and when click on next button
*/
void gtkcb_DebugEmuNextOp(GtkWidget* widget, gpointer data) {
	debug_op = 1;
	MipsCpu.halt = 2;
}


/*
When user see the debug window
and when click on check box Opcodes
*/
void gtkcb_DbgWinOpcode(GtkWidget* widget, gpointer data) {  

    if ( debug_op == 0 ) debug_op = 1;
    else debug_op = 0;

}

/*
When user see the debug window
and when click on check box DMA
*/
void gtkcb_DbgWinDMA(GtkWidget* widget, gpointer data) {
 
    if ( debug_dma == 0 ) debug_dma = 1;
    else debug_dma = 0;

}

/*
When user see the debug window
and when click on check box Syscall/BIOS
*/
void gtkcb_DbgWinBIOS(GtkWidget* widget, gpointer data) {
 
    if ( debug_print_syscall == 0 ) debug_print_syscall = 1;
    else debug_print_syscall = 0;

}



/*************************************************************************/
/*					GTK:GS CONFIGURATION WINDOW							 */
/*************************************************************************/
/*
When user click on "config/GS" Menu
*/
void    gtkcb_GSConf       (GtkWidget* widget, gpointer data) {     (*GSconfigure)(); }


/*************************************************************************/
/*					GTK:INPUT CONFIGURATION WINDOW							 */
/*************************************************************************/
/*
When user click on "config/GS" Menu
*/
void    gtkcb_InputConf       (GtkWidget* widget, gpointer data) {     (*PAD1configure)(); }






/*************************************************************************/
/*													 					 */
/*************************************************************************/
/*
null idle Function
*/
int nSX2Idle_nullfunc(GtkWidget *pWidget, gpointer pData) { return TRUE; }


_s32   hWND;



/*************************************************************************/
/*					EMU:INIT FUNCTION				 					 */
/*************************************************************************/

/*
idle Function
Emulation Initialisation
Function calls when the gui do nothing

In practice,
This function is call 11 times according the nsx2InitState variable
Each time the function is called, a part of the the switch/case is executed

*/
int nSX2InitProc  (GtkWidget *pWidget, gpointer pData ) {
	char 		tempText[100];		//for displaying text to the widget
	static int 	nsx2InitState = 0;	//Function State
	static int	IsInitOK = 0;		//
	static int	GSAlreadyLoad = FALSE;	//	
	int 		result;				//fonction return
	int 		boucle;				//loop
	char 		GSMainTitle[256];	//main windows title	

	GtkTextBuffer* text_buffer=0;
	GtkTextIter end_text_buffer,start_text_buffer;

	switch ( nsx2InitState ) {
		/* Print some infos */
		case 0:	{
			
			/*	Erase the EmuOutput Text  	*/
            text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(EmuOutput));
            gtk_text_buffer_get_end_iter (text_buffer,&end_text_buffer);
            gtk_text_buffer_get_start_iter (text_buffer,&start_text_buffer);
			gtk_text_buffer_delete(text_buffer, &start_text_buffer, &end_text_buffer);
 		           

   			sprintf(tempText,"nSX2 %1.1d.%2.2d.%1.1d",  nSX2_VERSION_MAJ, nSX2_VERSION_MIN,nSX2_VERSION_BUILD);
			PrintToEmuOutput(tempText);	
			
			sprintf(tempText,"|  web  : http://nsx2.emulation64.com");
			PrintToEmuOutput(tempText);	
			
			sprintf(tempText,"|_ mail : atreides.muaddib@caramail.com");
			PrintToEmuOutput(tempText);		
	
			break;
		}
		
		/*Open the elf File */
		case 1: {
			result = ElfOpenFile(ElfToExecute);	
			if ( result == FALSE )  IsInitOK++;
			break;			
		}
		
		/*Open the BIOS File  :	NOT USE BIOS FILE NOW	*/
		case 2: {
			if ( USE_PS2BIOS == 1 )	result = BiosOpenFile( "bios/scph10000.bin" ) ;		
			if ( result == FALSE )  IsInitOK++;
			
			break;		
		}
		
		/*Various Memory Allocation */
		case 3: {
			/* if memory was already allocated, free it */
			FreeAllMemory();
			
    		result = MipsMemoryAlloc();							if ( result == FALSE )  IsInitOK++;
			result = MipsSpecialAreaMemoryAlloc();				if ( result == FALSE )  IsInitOK++;
			result = MipsGSAreaMemoryAlloc();					if ( result == FALSE )  IsInitOK++;
			result = MipsBiosMemoryAlloc();						if ( result == FALSE )  IsInitOK++;
			break;
		}
	
		
		/*If something wrong with initialisation*/
		case 4: {
			if ( IsInitOK != 0 ) {
				sprintf(tempText,"FATAL ERROR while emulator initialisation....Sorry Exiting");	
				PrintToEmuOutput(tempText);
				gtkErrorEmulator(pWidget, NULL);
			}
			break;
		}
	
		/*Read the header of the Elf File*/
		case 5: {
			ElfLoadHeader();
			
			//ElfPrintInfo();
			break;
		}
	
		/*Copy the program to memory according the elf header*/
		case 6: {	
			CopyProgramToMemory() ;
			break;
		}
		
		/*Copy data to memory according the elf header*/
		case 7: {	
			CopySectionToMemory( );
			break;
		}
		
		/*Copy Bios to Memory */
		case 8: {
			if ( USE_PS2BIOS == 1 ) CopyBiosToMemory ( );	
			break;	
		}
	
		/* Initialisation of GS plugin */
		case 9: {	
			if ( GSAlreadyLoad != FALSE ) {
				
    			(*GSclose)();
            	(*GSshutdown)();
            	(*GSinit)();

   				(*PAD1close)();
            	(*PAD1shutdown)();
            	(*PAD1init)(1);
				

			}
			else GSAlreadyLoad = TRUE;
			
			result = (*GSopen)(&hWND ,"Video Output" );
		
			if ( result != 0 ) {
				sprintf(tempText,"FATAL ERROR while graphics initialisation...Sorry Exiting");
				PrintToEmuOutput(tempText);
				gtkErrorEmulator(pWidget, NULL);
				
			} else {
				sprintf(tempText,"GS:OK");
				PrintToEmuOutput(tempText);			
			}

			result = (*PAD1open)(&hWND);
		
			if ( result != 0 ) {
				sprintf(tempText,"FATAL ERROR while Input initialisation...Sorry Exiting");
				PrintToEmuOutput(tempText);
				gtkErrorEmulator(pWidget, NULL);
				
			} else {
				sprintf(tempText,"Input:OK");
				PrintToEmuOutput(tempText);			
			}
			break;	
		}

		/*Various PS2 Initialisation */
		case 10: {	
			
			/* Need To refresh at least time the display */
			Ps2Bios.NeedUpdateScreen 			= TRUE;
			
			/* Don't need to read Pad Status until I need it */
			Ps2Bios.NeedUpdateScreen 			= FALSE;
			
			/* Timer Init */
   			TimerInit(NSX2_TIMER);
   			Ps2Bios.AccesToGS = FALSE;
			
			/* r5900 Init */
			MipsCpu.PC 							= MipsCpu.EntryPointAdd ;	//Start Program At Addr
			MipsCpu.GPR[CPU_REG_GP].lo			= 0x81F80000;				//GP
			MipsCpu.GPR[CPU_REG_SP].lo			= 0x81F00000;				//SP
			MipsCpu.CPR0[12][0]					= 0x10900001;				//Reg Status of COP0
			MipsCpu.CPR0[15][0]					= 0x2E20	;				//Reg PrID of COP0
			MipsCpu.Nb_instructions 			= 0;

			MipsCpu.GPR[CPU_REG_R0].lo			= 0x0;
			
			/* Bios / Kernel Init */
			for ( result = 0; result < BIOS_MAX_SEMAPHORE; result++ ) Ps2Bios.SemaState[result] = BIOS_SEMA_ROOMFREE ;
			for ( result = 0; result < BIOS_MAX_INTC_HANDLER; result++ ) Ps2Bios.IntCH[ result ].Status = 0 ;
			for ( result = 0; result < BIOS_MAX_DMA_HANDLER; result++ ) Ps2Bios.DmaH[ result ].Status = 0 ;
			for ( result = 0; result < BIOS_MAX_THREADS; result++ ) Ps2Bios.ThreadState[result] = BIOS_THREAD_ROOMFREE ;

			/* Declare at least One Thread */
			Ps2Bios.ThreadState[0] = BIOS_THREAD_ROOMBUSY ;


			
			/*
			Create some pointers to val for performing the conversion by value of :
			 int -> float  and    float -> int
			without change the way of store
			for exemple :
				0x4863D000 = 233280.0000
				and not
				0x4863D000 = 1214500864.0000
			*/
			for ( result = 0 ; result < 32 ; result++ ) {
			     MipsCpu.FPR1_p_float[result]=(float *)&(MipsCpu.FPR1[result].lo);
             }

			*(MipsCpu.FPR1_p_float[0]) = (float)0.0;

			/* VU 0 Init */
			VectorUnit0.CCR[ VU_STATUS_FLAG ] 		= 0;
			VectorUnit0.CCR[ VU_MAC_FLAG ] 			= 0;
			VectorUnit0.CCR[ VU_CLIPPIN_FLAG ] 		= 0;
			VectorUnit0.CCR[ VU_FBRST_REGISTER ] 	= 0;
			VectorUnit0.CCR[ VU_VPU_STAT_RGISTER ] 	= 0;

			for ( boucle = 0 ; boucle < 4 ; boucle ++ ) {
				for ( result = 0 ; result < 32 ; result++ ) {
			     	VectorUnit0.CPR_p_float[result][boucle]=(float *)&(VectorUnit0.CPR[result][boucle]);
                 }
			}
			*(VectorUnit0.CPR_p_float[0][0]) = (float)0.0;
			*(VectorUnit0.CPR_p_float[0][1]) = (float)0.0;
			*(VectorUnit0.CPR_p_float[0][2]) = (float)0.0;
			*(VectorUnit0.CPR_p_float[0][3]) = (float)1.0;

			for ( boucle = 0 ; boucle < 32 ; boucle ++ ) {
			     	VectorUnit0.CCR_p_float[boucle]=(float *)&(VectorUnit0.CPR[boucle]);
 			}

			for ( boucle = 0 ; boucle < 4 ; boucle ++ ) {
			     	VectorUnit0.ACC_p_float[boucle]=(float *)&(VectorUnit0.ACC[boucle]);
 			}


			/* Pad Initialisation */
			Ps2Bios.PadData[0][0] = NULL;
			Ps2Bios.PadData[0][1] = NULL;
			Ps2Bios.PadData[1][0] = NULL;
			Ps2Bios.PadData[1][1] = NULL;
			
			Ps2Bios.PadState[0] = NULL;
			Ps2Bios.PadState[1] = NULL;
		
			break;
			
		}	

		/* Emulation Start Here...*/
		case 11: {
			/*Change the Idle proc of the Gui*/
            gtk_idle_remove    (nSX2Idle_funcID);
			nSX2Idle_funcID = gtk_idle_add( (GtkFunction)nSX2MainEmuProc ,  NULL );
           	
           	/*The Cpu is running*/
           	MipsCpu.halt = 0;
           	
           	/*For the next initialisation*/
			nsx2InitState = -1;
			IsInitOK = 0;

			/*Start a Timer*/
            GTimer_IPS = g_timer_new ();

			
			sprintf(tempText,"Start Interpreter....");
			PrintToEmuOutput(tempText);	
			break;
		}
		
		/* MUST NEVER DO THIS */
		default: {
			sprintf(tempText,"Too Bad Muad...");
			PrintToEmuOutput(tempText);	
			break;
		}
		
	}
	
	/* next State when calling again this fucntion */
	nsx2InitState ++;
	
	return TRUE;
}




/*************************************************************************/
/*					EMU:MAIN LOOP FUNCTION			 					 */
/*************************************************************************/
/*
idle Function
Main Function of the EMU
Function calls when the gui do nothing
*/
int nSX2MainEmuProc (GtkWidget *pWidget, gpointer pData ) {
char tempText[100];
	_u16	boucle;
	/*
	Old Version Debugging method
	*/
	/*
	if ( MipsCpu.PC == 0x00404158) debug_op = 1;				//...after acessing a opcode
	if ( MipsCpu.Nb_instructions > 557000 ) debug_op = 1;		//...after a number of opcode
	
	if ( MipsCpu.PC == 0x00202930 ) result++;	
	if ( result == 1 ) debug_op = 1;	
	*/		
	
	
	/*
		If the debug mode is running, I can't exec a block of opcode
	
	*/
	switch( MipsCpu.halt ) {

		/* the CPU is running */
		case 0: 

			if ( debug_op == 0 ) {
				for ( boucle = 0 ; boucle < 1000 ; boucle ++ ) { 
    				MipsUpdateCpu(); 
        			if ( MipsCpu.halt != 0) break;
       				} //Execute a Block of opcodes
			} else MipsUpdateCpu();												//Execute the current opcode
			
		break;
		
		/* the CPU is halted */
		case 1:	break;
		
		/* the CPU execute ONE opcode and Stop */
		case 2:
			MipsUpdateCpu();	//Execute the current opcode
			MipsCpu.halt = 1;	//And Stop
		break;
		
		
	}

	/*Execute DMA stuff*/
	DmaUpdate();
	
	/*Execute GS stuff*/
	GSUpdate();

	/* Execute Handlers */
	BiosUpdate();
	
	/* Update Timer and Main Clock */
	TimerUpdate();


	/* Pad Upddate and Input Keyboard */		
	PadAndKeyUpdate();

	
	/* In future add other stuff like  SoundUpdate , etc.... */
	
	
    ComputeInstructionsPerSec();
	
	return TRUE;

}


/*************************************************************************/
/*					EMU:END FUNCTION				 					 */
/*************************************************************************/
/*
When Exiting App,
*/
void arreter_boucle(GtkWidget* widget)
{
  	/*
  	char tempText[100];
	PrintToEmuOutput("...Stop Interpreter");	
	sprintf(tempText,"Stop at PC = 0x%8.8llx after %lld instructions",MipsCpu.PC, MipsCpu.Nb_instructions);
	PrintToEmuOutput(tempText);	
	*/
	
	/* Remove the idle Function*/
	gtk_idle_remove    (nSX2Idle_funcID);

	/*Destroy the IPS Timer*/
    g_timer_destroy (GTimer_IPS);
    
    /*Shutdown GS Pluggin*/
	if ( GSshutdown != NULL ) (*GSshutdown)();
	if ( PAD1shutdown != NULL ) (*PAD1shutdown)();	

 	/*Close Files*/
	ElfCloseFile();
	if ( USE_PS2BIOS == 1 )BiosCloseFile();
 	
	/*Free All Emu Memory*/
	FreeAllMemory();

	/*Stop GTK Main Loop*/
	gtk_main_quit();
}



/*************************************************************************/
/*					EMU:							 					 */
/*************************************************************************/
/*

*/

float ComputeInstructionsPerSec( void  ) {

        static gdouble	PreviousTime = 0.0;
        gulong Timemicroseconds;
        static _u32	PreviousNbInstruction = 0;
        
        gdouble	CurrentTime;
        float	InstPerSec;
        
        char temp_string [100];
	
        CurrentTime =  g_timer_elapsed (GTimer_IPS, &Timemicroseconds);

		if ( CurrentTime < PreviousTime ) PreviousTime = 0.0;
	
		if ( ( CurrentTime - PreviousTime ) > 1.0 ) {


			InstPerSec = (float)(( MipsCpu.Nb_instructions - PreviousNbInstruction ) * ( ( CurrentTime - PreviousTime ) )) / 1000000.0;

			if ( InstPerSec > (float)500.0 ) {
				sprintf(temp_string,"CPU Speed: ?\n");
			} else {
				if ( InstPerSec == (float)0.0 ) {
					sprintf(temp_string,"Emulation Stopped");
				} else {
					sprintf(temp_string,"CPU Speed:%10.6f MI / s\n",InstPerSec);
				}
			}

			gtk_statusbar_push (GTK_STATUSBAR (StatusBar), GPOINTER_TO_INT(ContextId1), g_locale_to_utf8(temp_string, -1, NULL, NULL, NULL));
	
			PreviousNbInstruction = MipsCpu.Nb_instructions;
			PreviousTime = 	CurrentTime;
		}

}


/*************************************************************************/
/*					EMU: Log Function				 					 */
/*************************************************************************/
void PrintToEmuOutput (  char * ftext ) {
	
	GtkTextBuffer* text_buffer=0;
	GtkTextIter end;
	GtkTextIter LastLine;
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(EmuOutput));
	gtk_text_buffer_get_end_iter(text_buffer,&end);

	
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8(ftext, -1, NULL, NULL, NULL), -1);
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8("\n", -1, NULL, NULL, NULL), -1);

	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(EmuOutput));
	gtk_text_buffer_get_end_iter(text_buffer,&LastLine);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(EmuOutput), &LastLine, 0.0, FALSE, 0.0, 0.0);
	

}

void PrintToEmuOutputPS2Output (  char * ftext ) {
	static _u8	gtk_AlreadyCreated = FALSE;
    static GtkTextTag *tag;

	GtkTextBuffer* text_buffer;
	GtkTextIter end,start;
	GtkTextIter LastLine;
	
	text_buffer = gtk_text_view_get_buffer( GTK_TEXT_VIEW(EmuOutput) );
	gtk_text_buffer_get_end_iter	( text_buffer, &end );

	/* Use a tag to change the color for just one part of the widget */
  	if ( gtk_AlreadyCreated == FALSE ) {
		tag = gtk_text_buffer_create_tag (text_buffer, "blue_foreground", "foreground", "white", "background", "black",NULL);  
		gtk_AlreadyCreated = TRUE;
	}

	gtk_text_buffer_insert_with_tags(text_buffer, &end, g_locale_to_utf8(ftext, -1, NULL, NULL, NULL), -1, tag, NULL);	

	gtk_text_buffer_get_end_iter(text_buffer,&LastLine);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(EmuOutput), &LastLine, 0.0, FALSE, 0.0, 0.0);


}


void PrintToOpcodeOutput (  char * ftext ) {
	
	GtkTextBuffer* text_buffer=0;
	GtkTextIter end;
	GtkTextIter LastLine;
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewOpcodes));
	gtk_text_buffer_get_end_iter(text_buffer,&end);

	
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8(ftext, -1, NULL, NULL, NULL), -1);
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8("\n", -1, NULL, NULL, NULL), -1);	

	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewOpcodes));
	gtk_text_buffer_get_end_iter(text_buffer,&LastLine);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(textviewOpcodes), &LastLine, 0.0, FALSE, 0.0, 0.0);
	

}

void PrintToDmaOutput (  char * ftext ) {
	
	GtkTextBuffer* text_buffer=0;
	GtkTextIter end;
	GtkTextIter LastLine;
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewDMA));
	gtk_text_buffer_get_end_iter(text_buffer,&end);

	
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8(ftext, -1, NULL, NULL, NULL), -1);
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewDMA));
	gtk_text_buffer_get_end_iter(text_buffer,&LastLine);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(textviewDMA), &LastLine, 0.0, FALSE, 0.0, 0.0);
	

}

void PrintToBIOSOutput (  char * ftext ) {
	
	GtkTextBuffer* text_buffer=0;
	GtkTextIter end;
	GtkTextIter LastLine;
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewBIOS));
	gtk_text_buffer_get_end_iter(text_buffer,&end);

	
	gtk_text_buffer_insert(text_buffer, &end, g_locale_to_utf8(ftext, -1, NULL, NULL, NULL), -1);
	
	text_buffer=gtk_text_view_get_buffer(GTK_TEXT_VIEW(textviewBIOS));
	gtk_text_buffer_get_end_iter(text_buffer,&LastLine);
	gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW(textviewBIOS), &LastLine, 0.0, FALSE, 0.0, 0.0);
	

}



void PrintToRegistersOutput (  char * ftext ) {
	char TempText[128];



	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_R0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_R0].lo));
	gtk_label_set_text (GTK_LABEL(val_r0), g_locale_to_utf8(TempText, -1, NULL, NULL, NULL));
	

	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_AT].lo>>32),(long)(MipsCpu.GPR[CPU_REG_AT].lo));
	gtk_label_set_text (GTK_LABEL(val_at), g_locale_to_utf8(TempText, -1, NULL, NULL, NULL));


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_V0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_V0].lo));
	gtk_label_set_text (GTK_LABEL(val_v0), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_V1].lo>>32),(long)(MipsCpu.GPR[CPU_REG_V1].lo));
	gtk_label_set_text (GTK_LABEL(val_v1), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_A0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_A0].lo));
	gtk_label_set_text (GTK_LABEL(val_a0), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_A1].lo>>32),(long)(MipsCpu.GPR[CPU_REG_A1].lo));
	gtk_label_set_text (GTK_LABEL(val_a1), TempText);

	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_A2].lo>>32),(long)(MipsCpu.GPR[CPU_REG_A2].lo));
	gtk_label_set_text (GTK_LABEL(val_a2), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_A3].lo>>32),(long)(MipsCpu.GPR[CPU_REG_A3].lo));
	gtk_label_set_text (GTK_LABEL(val_a3), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T0].lo));
	gtk_label_set_text (GTK_LABEL(val_t0), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T1].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T1].lo));
	gtk_label_set_text (GTK_LABEL(val_t1), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T2].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T2].lo));
	gtk_label_set_text (GTK_LABEL(val_t2), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T3].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T3].lo));
	gtk_label_set_text (GTK_LABEL(val_t3), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T4].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T4].lo));
	gtk_label_set_text (GTK_LABEL(val_t4), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T5].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T5].lo));
	gtk_label_set_text (GTK_LABEL(val_t5), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T6].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T6].lo));
	gtk_label_set_text (GTK_LABEL(val_t6), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T7].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T7].lo));
	gtk_label_set_text (GTK_LABEL(val_t7), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S0].lo));
	gtk_label_set_text (GTK_LABEL(val_s0), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S1].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S1].lo));
	gtk_label_set_text (GTK_LABEL(val_s1), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S2].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S2].lo));
	gtk_label_set_text (GTK_LABEL(val_s2), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S3].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S3].lo));
	gtk_label_set_text (GTK_LABEL(val_s3), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S4].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S4].lo));
	gtk_label_set_text (GTK_LABEL(val_s4), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S5].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S5].lo));
	gtk_label_set_text (GTK_LABEL(val_s5), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S6].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S6].lo));
	gtk_label_set_text (GTK_LABEL(val_s6), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S7].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S7].lo));
	gtk_label_set_text (GTK_LABEL(val_s7), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T8].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T8].lo));
	gtk_label_set_text (GTK_LABEL(val_t8), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_T9].lo>>32),(long)(MipsCpu.GPR[CPU_REG_T9].lo));
	gtk_label_set_text (GTK_LABEL(val_t9), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_K0].lo>>32),(long)(MipsCpu.GPR[CPU_REG_K0].lo));
	gtk_label_set_text (GTK_LABEL(val_k0), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_K1].lo>>32),(long)(MipsCpu.GPR[CPU_REG_K1].lo));
	gtk_label_set_text (GTK_LABEL(val_k1), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_GP].lo>>32),(long)(MipsCpu.GPR[CPU_REG_GP].lo));
	gtk_label_set_text (GTK_LABEL(val_gp), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_SP].lo>>32),(long)(MipsCpu.GPR[CPU_REG_SP].lo));
	gtk_label_set_text (GTK_LABEL(val_sp), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_S8].lo>>32),(long)(MipsCpu.GPR[CPU_REG_S8].lo));
	gtk_label_set_text (GTK_LABEL(val_s8), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_RA].lo>>32),(long)(MipsCpu.GPR[CPU_REG_RA].lo));
	gtk_label_set_text (GTK_LABEL(val_ra), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_HI].lo>>32),(long)(MipsCpu.GPR[CPU_REG_HI].lo));
	gtk_label_set_text (GTK_LABEL(val_hi), TempText);


	sprintf(TempText,"%8.8lx-%8.8lx",(long)(MipsCpu.GPR[CPU_REG_LO].lo>>32),(long)(MipsCpu.GPR[CPU_REG_LO].lo));
	gtk_label_set_text (GTK_LABEL(val_lo), TempText);

} 

void PrintToCOP1RegistersOutput (  char * ftext ) {
	
 	char TempText[128];
 	
	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 0 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp00), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 1 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp01), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 2 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp02), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 3 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp03), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 4 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp04), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 5 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp05), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 6 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp06), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 7 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp07), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 8 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp08), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 9 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp09), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 10 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp10), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 11 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp11), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 12 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp12), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 13 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp13), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 14 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp14), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 15 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp15), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 16 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp16), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 17 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp17), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 18 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp18), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 19 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp19), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 20 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp20), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 21 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp21), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 22 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp22), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 23 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp23), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 24 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp24), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 25 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp25), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 26 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp26), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 27 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp27), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 28 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp28), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 29 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp29), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 30 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp30), TempText);

	sprintf(TempText,"%12.6f",*(MipsCpu.FPR1_p_float[ 31 ]) );
	gtk_label_set_text (GTK_LABEL(val_fp31), TempText);

	
} 

  
void PrintExtraStuff( void ) {
	char TempText[256];

	sprintf(TempText,"%8.8llx",MipsCpu.PC);
	gtk_label_set_text (GTK_LABEL(val_PC), TempText);
	sprintf(TempText,"%ld%ld",(long)(MipsCpu.Nb_instructions>>32),(long)(MipsCpu.Nb_instructions));
	gtk_label_set_text (GTK_LABEL(val_InstrNumber), TempText);



}


