/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/


/*

<Shadowpri>  Topic is 'http://www.nsx2.tk/ | Muad send you suggestions to what_daa@hotmail.com (myth)

linuzappz@pcsx.net for input plugin

*/
/********************************************************************************************
							GENERAL EMULATOR DEFINITION
*********************************************************************************************/

#ifndef _NSX2_H
#define _NSX2_H


#include <glib.h>




#define nSX2_VERSION_MAJ	0
#define nSX2_VERSION_MIN	6
#define nSX2_VERSION_BUILD	00
 


#define	nSX2ConfigFile	"./nsx2.ini"

#define nSX2DemoDir		"./demos/"

/*
Use the GS pluggin
*/
#define PLUGINtypedefs
#define PLUGINfuncs




/*
Set to 1, for use scph10000.bin
*/
#define	USE_PS2BIOS		0



/********************************************************************************************
							TYPE DEFINITION
*********************************************************************************************/
#ifndef TRUE

#define TRUE 	1
#define FALSE 	0

#endif

typedef unsigned long long	_u64;
typedef unsigned long		_u32;
typedef unsigned short		_u16;
typedef unsigned char		_u8;

typedef	long long			_s64;
typedef	long				_s32;
typedef	short				_s16;
typedef	char				_s8;

/********************************************************************************************
							EMULATOR DEFINES
*********************************************************************************************/

#define CPU_REG_R0	0
#define CPU_REG_AT	1
#define CPU_REG_V0	2
#define CPU_REG_V1	3
#define CPU_REG_A0	4
#define CPU_REG_A1	5
#define CPU_REG_A2	6
#define CPU_REG_A3	7
#define CPU_REG_T0	8
#define CPU_REG_T1	9
#define CPU_REG_T2	10
#define CPU_REG_T3	11
#define CPU_REG_T4	12
#define CPU_REG_T5	13
#define CPU_REG_T6	14
#define CPU_REG_T7	15
#define CPU_REG_S0	16
#define CPU_REG_S1	17
#define CPU_REG_S2	18
#define CPU_REG_S3	19
#define CPU_REG_S4	20
#define CPU_REG_S5	21
#define CPU_REG_S6	22
#define CPU_REG_S7	23
#define CPU_REG_T8	24
#define CPU_REG_T9	25
#define CPU_REG_K0	26
#define CPU_REG_K1	27
#define CPU_REG_GP	28
#define CPU_REG_SP	29
#define CPU_REG_S8	30
#define CPU_REG_RA	31
#define CPU_REG_HI	32
#define CPU_REG_LO	33

#define SIGN_BIT_MASK       0x80000000
#define SIGN_BIT_MASK_64    0x8000000000000000

/*
COP 1 Control Register
*/
#define CP1_FIR		0	//Floating-Point Implementation Register (FIR, CP1 Control Register 0)
#define CP1_FCCR	25	//Floating-Point Condition Codes Register (FCCR, CP1 Control Register 25)
#define CP1_FEXR	26	//Floating-Point Exceptions Register (FEXR, CP1 Control Register 26)
#define CP1_FENR	28	//Floating-Point Enables Register (FENR, CP1 Control Register 28)
#define CP1_FCSR	31	//Floating-Point Control and Status Register (FCSR, CP1 Control Register 31)

#define CP1_FCSR_C_MASK     0x00800000
#define CP1_FCSR_C_NOT_MASK 0xFF7FFFFF


/********************************************************************************************
							STRUCTURES DEFINITIONS
*********************************************************************************************/


/*
128 bits structure
*/
typedef struct __u128 {
	_u64 lo;
	_u64 hi;
} _u128;


/*
Main CPU R5900
*/
typedef	struct _Mips_CPU {
	_u32		opcode;					//current opcode
	_u8			halt;					//if 1 stop the emu
	
	_u128 		GPR[34];				//General Registers
	
	_u64		CPR0[32][4];			//Cop0	Control Register

	_u128		FPR1[32];				//COP1	FloatingPoint Register for the CPU
    float		*FPR1_p_float[32];		//COP1	Pointer Float for conversion
	
	
	_u64		CP1[32];				//COP1 Control Register
	
	_u64		Nb_instructions;		//Number Of Instruction Exececuted
	_u64		PC;						//Program Counter
	_u32 		EntryPointAdd;			//Start the emu at
	
	
	/* Interpreter V2 */
	
	
} Mips_CPU;

/* COP 2 - VU */

#define		VU_STATUS_FLAG		16
#define		VU_MAC_FLAG			17	
#define		VU_CLIPPIN_FLAG		18
#define		VU_R_REGISTER		20
#define		VU_I_REGISTER		21
#define		VU_Q_REGISTER		22
#define		VU_TPC_REGISTER		26
#define		VU_CMSAR0_REGISTER	27
#define		VU_FBRST_REGISTER	28
#define		VU_VPU_STAT_RGISTER	29
#define		VU_CMSAR1_REGISTER	31


typedef struct	_VPU0 {

	_u32		CPR[32][4];				//VPU0	FloatingPoint Register for the CPU
    float		*CPR_p_float[32][4];	//VPU0	Pointer Float for conversion

	_u32		ACC[4];					//VPU0	
	float		*ACC_p_float[4];		//VPU0	

	_u32		CCR[32];				//VPU0	IntegerRegister	&
										// 		Control Register
	
	float		*CCR_p_float[32];	//VPU0	Pointer Float for conversion

}VPU0;



/*
DMA Channel Definition
*/
typedef struct _DmaChannel {

	_u64		ctrl;			//Channel Control register
	_u64		memAdd;			//Memory address
	_u64		size;			//Quadra-word counter
	_u64		tag;			//Tag address
	_u64		asr0;			//Assembler Reserved 0
	_u64		asr1;			//Assembler Reserved 1
	_u64		saddr;			//???

} DmaChannel;

/*
DMA Channel Controler
*/
typedef struct _DmaChannelControler {

	_u64		ctrl;			//DMAC Control
	_u64		stat;			//DMAC Status
	_u64		pcr;			//DMAC Priority control
	_u64		sqwc;			//DMAC skip quad word
	_u64		rbsr;			//DMAC ring buffer size
	_u64		rbor;			//DMAC Ring buffer offset
	_u64		stadr;			//DMA stall address

} DmaChannelControler;




/*
DMA HANDLER
*/

typedef struct	_DmaHandler {
	
	_u32		Status;			//Dma Handler Status
	_u64		Add;			//Adress of the Handler
	_u32		ID;				//

/* Maybe Delete this Entry */
	_u8			Enable;			//Enable or Disable the Handler 
  	_u32        EndAdd;
    _u64		ReturnAdd;		//Return Add
	_u8			Running;		//???
		
} DmaHandler;

/*
INT C HANDLER
*/

typedef struct	_IntCHandler {
	
	_u32		Status;			//Dma Handler Status
	_u64		Add;			//Adress of the Handler
	_u32		ID;				//
		
} IntCHandler;

/*
SEMAPHORE
*/
typedef struct _Bios_Sema {
    
    _u64    MemAddress;

    _s32 currentCount;              //Current semaphore count
    _s32 maxCount;                  //Maximum semaphore count
    _s32 initCount;                 //Initial semaphore count
    _s32 numWaitThreads;            //Number of threads waiting on semaphore
    _s32 attr;                      //Semaphore attribute
    _u32 option;                   	//
    
} Bios_Sema;

/*
Threads
*/
typedef struct _Bios_Thread {
	_s32 status; 				//Thread status
	_u32 entry_add;				//void (*entry)(void *); Entry address for execution 	(*)
	_u32 stack_add;				//void *stack; Stack address used by thread 			(*) (16-byte alignment)
	_s32 stackSize; 			//Stack size (in bytes: multiple of 16) 				(*)
	_u32 gpReg_add;				//void *gpReg; Value of GP (Global Pointer) register 	(*)
	_s32 initPriority;			//Initial priority (0 - 128) 							(*)
	_s32 currentPriority;		//Current priority (0 - 128)
	_u32 attr; 					//System Reserved (value is arbitrary)
	_u32 option; 				//Optional user-defined information
	_s32 waitType; 				//WAIT type
	_s32 waitId;				//semaphore ID if WAIT type is "semaphore"
	_s32 wakeupCount; 			//Wakeup request count

	_u128   GPR[34];
	_u128	FPR1[32];
	
	
}Bios_Thread; 

/*
Timers
*/

typedef struct _HW_Timer_mode {
	_u8	CLKS;	//Clock Selection
	_u8	GATE;	//Gate Function Enable
	_u8	GATS;	//Gate Selection
	_u8	GATM;	//Gate Mode
	_u8 ZRET;	//Zero Return
	_u8 CUE;	//Count Up Enable
	_u8 CMPE;	//Compare-Interrupt Enable
	_u8	OVFE;	//Overflow-Interrrupt Enable
	_u8	EQUF;	//Equal Flag
	_u8	OVFF;	//Overflow Flag
	
}HW_Timer_mode;

typedef struct _HW_Timer {
	_u64	mode;		//Mode Setting and Status reading
	_u64	count;		//Counter value ( the upper 16 bits are fixed to 0 )
	_u64	comp;		//Compare value ( the upper 16 bits are fixed to 0 )
	_u64	hold;		//Hold value ( the upper 16 bits are fixed to 0 )
	
	HW_Timer_mode mode_param;
	
}HW_Timer;


/*
Pad
*/

/*
 * Pad states
 */
#define PAD_STATE_DISCONN     	0x00
#define PAD_STATE_FINDPAD       0x01
#define PAD_STATE_FINDCTP1      0x02
#define PAD_STATE_EXECCMD       0x05
#define PAD_STATE_STABLE        0x06
#define PAD_STATE_ERROR      	0x07



/*
 * Pad request states
 */
#define PAD_RSTAT_COMPLETE      0x00
#define PAD_RSTAT_FAILED        0x01
#define PAD_RSTAT_BUSY          0x02



#define PAD_RPCCMD_OPEN         0x80000100
#define PAD_RPCCMD_SET_MMODE    0x80000105
#define PAD_RPCCMD_SET_ACTDIR   0x80000106
#define PAD_RPCCMD_SET_ACTALIGN 0x80000107
#define PAD_RPCCMD_GET_BTNMASK  0x80000108
#define PAD_RPCCMD_SET_BTNINFO  0x80000109
#define PAD_RPCCMD_SET_VREF     0x8000010a
#define PAD_RPCCMD_GET_PORTMAX  0x8000010b
#define PAD_RPCCMD_GET_SLOTMAX  0x8000010c
#define PAD_RPCCMD_CLOSE        0x8000010d
#define PAD_RPCCMD_END          0x8000010e


/*
typedef struct _Pad_State
{
    int open;
    unsigned int port;
    unsigned int slot;
    struct pad_data *padData;
    unsigned char *padBuf;
}Pad_State;
*/

typedef struct _Pad_Data 
{
    unsigned int frame;			//0-3
    unsigned char state;		//4
    unsigned char reqState;		//5
    unsigned char ok;			//6
    unsigned char unkn7;		//7
    unsigned char data[32];		//8-39
    unsigned int length;		//40-43
    unsigned int unkn44;		//44-47
    unsigned int unkn48;		//48		: Pad Type
    unsigned int unkn52;
    unsigned int unkn54;
    unsigned int unkn60;
}Pad_Data ;

typedef struct _padButtonStatus
{
    unsigned char ok;			//0
    unsigned char mode;			//1
    unsigned char btns[2];		//2-3
    // joysticks
    unsigned char rjoy_h;		//4
    unsigned char rjoy_v;		//5
    unsigned char ljoy_h;		//6
    unsigned char ljoy_v;		//7
    // pressure mode
    unsigned char left_p;		//8
    unsigned char rigth_p;		//9
    unsigned char up_p;			//10
    unsigned char down_p;		//11
    unsigned char triangle_p;	//12
    unsigned char circle_p;		//13
    unsigned char square_p;		//14
    unsigned char cross_p;		//15
    unsigned char unkn16[16];	//16-31
}padButtonStatus;


#define	PAD_INFO_TYPE_NEGI_COM 		2
#define	PAD_INFO_TYPE_KONAMI_GUN 	3
#define	PAD_INFO_TYPE_STANDARD  	4
#define	PAD_INFO_TYPE_JOY_STICK 	5
#define	PAD_INFO_TYPE_NAMCO_GUN 	6
#define	PAD_INFO_TYPE_ANALOG  		7
#define	PAD_INFO_TYPE_TSURI_CON 	0x100:
#define	PAD_INFO_TYPE_JOG_CON 		0x300:


/*
Internal Emulator BIOS
*/

#define		BIOS_MAX_SEMAPHORE		32
#define		BIOS_SEMA_ROOMFREE		0
#define		BIOS_SEMA_ROOMBUSY		1


#define		BIOS_MAX_DMA_HANDLER	10
#define		BIOS_MAX_INTC_HANDLER	11

#define		BIOS_MAX_SIF_DCHAIN		256

#define		BIOS_MAX_THREADS			32
#define		BIOS_THREAD_ROOMFREE		0
#define		BIOS_THREAD_ROOMBUSY		1


#define		BIOS_MAX_DMA_CHANNEL	10


/*
This Structure define 
BIOS PS2 
and 
Other Hardware : DMA Controller, Timer, etc

*/
typedef	struct _Bios_Hw {

//	_u64		GsCrt;
	
	/*
	Detecting Acces to DMA Area Memory
	*/
	_u8		WriteToDmaArea;
	_u64	DmaAreaWriteAt;
	_u64	DmaAreaWriteVal;
	
	_u8		ReadToDmaArea;
	_u64	DmaAreaReadAt;
	_u64	DmaAreaReadVal;	


	/* DMA CHANNEL */
	DmaChannelControler DmaControler;
 	DmaChannel 			DmaC[BIOS_MAX_DMA_CHANNEL];
	
	/* DMA HANDLER */
	DmaHandler	DmaH[BIOS_MAX_DMA_HANDLER];


	/* INTC HANDLER */
	IntCHandler	IntCH[BIOS_MAX_INTC_HANDLER];


	/* SEMAPHORE STUFF */
	Bios_Sema	Sema		[BIOS_MAX_SEMAPHORE];
	_u8			SemaState	[BIOS_MAX_SEMAPHORE];

	
	/* THREADS STUFF*/
	Bios_Thread	Treads		[BIOS_MAX_THREADS];
	_u8			ThreadState	[BIOS_MAX_THREADS];	

	/* TIMERS */
	HW_Timer	Timer[5];


	/* Pads */
	Pad_Data 	* PadData[2][2];
	padButtonStatus 	* PadState[2];
	
	/* VERY BAD WAY*/
	_u64	NeedUpdateScreen;
	_u8		AccesToGS;

	_u8 	NeedUpdatePad;

} Bios_Hw;





/********************************************************************************************
							EXTERN GLOBALS VARIABLES
*********************************************************************************************/

extern FILE * ElfFile;		//Current Elf File
extern long ElfFileSize;	//Size of the Elf File
extern FILE * BiosFile;		//Current BIOS File


extern Mips_CPU MipsCpu;	//Main CPU
extern Bios_Hw  Ps2Bios;	//Internal BIOS

extern	VPU0	VectorUnit0;	//VU0 unit


extern _u8	*MipsMemory;			//Main Memory 32 Mbits
extern _u8	*SpecialAreaMemory;		//Special Memory ( PS2 Control Register  )
extern _u8	*GSAreaMemory;			//GS control register
extern _u8	*BiosMemory;			//Bios Memory

extern char ElfToExecute[256];		//Elf File to Execute




extern char 	GSPlugginToUse[256];
extern _u32  	(* PS2EgetLibTypeGS)();
extern _u32  	(* PS2EgetLibVersionGS)();
extern char*	(* PS2EgetLibNameGS)();


extern char 	InputPlugginToUse[256];
extern _u32  	(* PS2EgetLibTypePAD)();
extern _u32  	(* PS2EgetLibVersionPAD)();
extern char*	(* PS2EgetLibNamePAD)();

extern char 	SifPlugginToUse[256];




extern _u16		muGUI_MainScreenW;
extern _u16		muGUI_MainScreenH;
extern _u16		muGUI_MainScreenBPP;
extern _u16		muGUI_MainScreenStretch;


/********************************************************************************************
							MACROS DEFINITIONS
*********************************************************************************************/
  





/*******************************************************************************************/
void muGUI_nSX2Proc ( void ) ;
void muGUI_nSX2InitProc ( void ) ;
void muGUI_nSX2EndingProc ( void ) ;


/********************************************************************************************
							ELF FILE DECLARATIONS
*********************************************************************************************/
long 	FileSizeOf			( FILE *fp );
_u8 	ElfOpenFile			( char * fFileName );
void 	ElfCloseFile		(void);
_u8 	ElfLoadHeader		(void);
void 	ElfPrintInfo		(void);
void 	ElfPrintSectionInfo	( _u8 fIndex );

void 	BiosCloseFile		(void);
_u8 	BiosOpenFile		( char * fFileName );



/********************************************************************************************
							MEMORY DECLARATIONS
*********************************************************************************************/
#define DATA_QUAD		1		//128 bits	
#define DATA_DW			2		//64 bits	
#define DATA_DWH		32		//64 bits
#define DATA_W			4		//32 bits
#define DATA_HW			8		//16 bits
#define DATA_B			16		//8 bits

_u8 MipsMemoryAlloc				(void);
_u8 MipsSpecialAreaMemoryAlloc	(void);
_u8 MipsGSAreaMemoryAlloc		(void);
_u8 MipsBiosMemoryAlloc			(void);

void 	FreeAllMemory			(void);
void 	CopyProgramToMemory		(void);
void 	CopySectionToMemory 	(void);
void 	CopyBiosToMemory 		(void);

void	MipsMemoryWrite 	( _u128 fVal,	_u8 fDataType,	_u64 fBase, 	int fOffset );
void	MipsSpecialMemWrite	( _u8 * Mem,	_u128  fVal,	_u8 fDataType, 	_u64 fBase,	int fOffset );
void	MipsGSAreaMemWrite	( _u8 * Mem,	_u128  fVal,	_u8 fDataType, 	_u64 fBase,	int fOffset );
void  	MipsBiosAreaMemWrite( _u8 * Mem, 	_u128  fVal, 	_u8 fDataType, 	_u64 fBase, int fOffset );
void	MipsMemoryWriteDW 	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteW  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteHW  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );
void	MipsMemoryWriteB  	( _u8 * Mem,	_u64 fVal,		_u64 fAdd );

void	MipsMemoryRead 		( _u128 * fVal,	_u8 fDataType,	_u64 fBase,		int fOffset );
void	MipsSpecialMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );
void	MipsGSAreaMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );
void	MipsBiosAreaMemRead	( _u8 * Mem,	_u128 * fVal,	_u8 fDataType,	_u64 fBase,	int fOffset );

void 	MipsMemoryReadDQ 	( _u8 * Mem,  	_u128 * fVal , 	_u64 fAddr );
void	MipsMemoryReadDWH 	( _u8 * Mem,	_u128 * fVal,	_u64 PhysicalAdd );
void	MipsMemoryReadDW 	( _u8 * Mem,	_u128 * fVal,	_u64 PhysicalAdd );
void	MipsMemoryReadW 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );
void	MipsMemoryReadHW 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );
void	MipsMemoryReadB 	( _u8 * Mem,	_u128 * fVal,	_u64 fAddr );


/********************************************************************************************
							CONFIG FILE DECLARATIONS
*********************************************************************************************/
int 	nSX2LoadConfig 	(void);



/********************************************************************************************
							MAIN CPU R5900 DECLARATIONS
*********************************************************************************************/
void	MipsUpdateCpu 		(void);
void	MipsCpuPrefetchOpcode(void);

_u8		debug_op;
_u8		debug_regs;
_u8		debug_gs;
_u8		debug_dma;
_u8		debug_print_syscall;

/********************************************************************************************
							GS MAIN DECLARATIONS
*********************************************************************************************/
#define GS_PMODE 		0x12000000    		//Various PCRTC modes
#define GS_SMODE1 		0x12000010			//Related to Sync
#define GS_SMODE2 		0x12000020			//Related to Sync
#define GS_SRFSH 		0x12000030			//DRAM refresh
#define GS_SYNCH1 		0x12000040			//Related to Sync
#define GS_SYNCH2 		0x12000050			//Related to Sync
#define GS_SYNCV		0x12000060			//Related to Sync/start
#define GS_DISPFB1	 	0x12000070 			//Related to display buffer of Rectangular Area 1
#define GS_DISPLAY1 	0x12000080			//Rectangular Area 1 display position etc.
#define GS_DISPFB2 		0x12000090			//Related to display buffer of Rectangular Area 2
#define GS_DISPLAY2 	0x120000a0			//Rectangular Area 2 display position etc.
#define GS_EXTBUF 		0x120000b0			//Rectangular Area write buffer
#define GS_EXTDATA	 	0x120000c0			//Rectangular Area write data
#define GS_EXTWRITE 	0x120000d0			//Rectangular Area write start
#define GS_BGCOLOR 		0x120000e0			//Background color
#define GS_IMR			0x12001010			//IMR



void 	GSUpdate			(void);




/********************************************************************************************
							DMA DECLARATIONS
*********************************************************************************************/

#define DMA_CH0 	0		//Linked to VIF0
#define DMA_CH1 	1		//Linked to VIF1
#define DMA_CH2 	2		//Linked to GIF
#define DMA_CH3 	3		//Linked to IPU (FROM???)
#define DMA_CH4 	4		//Linked to IPU (TO???)
#define DMA_CH5 	5		//Linked to SIF0
#define DMA_CH6 	6		//Linked to SIF1
#define DMA_CH7 	7		//Linked to SIF2
#define DMA_CH8 	8		//Linked to SPR  (from SCRATCH PAD to RAM???)
#define DMA_CH9 	9		//Linked to SPR  (from RAM to SCRATCH PAD???)
#define DMA_CONTROL 10		


#define DMA_CTRL    0x1000E000          //DMAC Control
#define DMA_STAT    0x1000E010          //DMAC Status
#define DMA_PCR     0x1000E020          //DMAC Priority control
#define DMA_SQWC    0x1000E030          //DMAC skip quad word
#define DMA_RBSR    0x1000E040          //DMAC ring buffer size
#define DMA_RBOR    0x1000E050          //DMAC Ring buffer offset
#define DMA_STADR   0x1000E060          //DMA stall address


#define	DMA_5_CHCR	0x1000C000			//Ch5 Channel Control
#define	DMA_5_MADR	0x1000C010			//Ch5 Memory Address
#define	DMA_5_QWC	0x1000C020			//Ch5 Quad word count


extern _u64 nb_dma_transfert;

void	DmaUpdate			(void);
void 	DmaPrintChannelInfo ( _u8 fChannel );

/********************************************************************************************
							DMA GS DECLARATIONS
*********************************************************************************************/

/*
TAG SOURCE CHAIN MODE : ID/FLAG
*/
#define	DMA_SRC_CHAIN_MODE_REFE		0x0
#define	DMA_SRC_CHAIN_MODE_CONT		0x1
#define	DMA_SRC_CHAIN_MODE_NEXT		0x2
#define	DMA_SRC_CHAIN_MODE_REF		0x3
#define	DMA_SRC_CHAIN_MODE_REFS		0x4
#define	DMA_SRC_CHAIN_MODE_CALL		0x5
#define	DMA_SRC_CHAIN_MODE_RET		0x6
#define	DMA_SRC_CHAIN_MODE_END		0x7


//Dma Transfer Size in Byte ( 16 = 128 bits )
#define DMA_TRS_TAG_SIZE    		16

void DMA_GS_Transfer_Normal_Mode( void );
void DMA_GS_Transfer_SourceChain_Mode( void );
void DMA_GS_Transfer ( void );

/********************************************************************************************
							BIOS DECLARATIONS
*********************************************************************************************/

#define INTC_GS  			0
#define INTC_SBUS  			1
#define INTC_VBON  			2	/* old fashioned */
#define	INTC_VBLANK_S		2
#define INTC_VBOF  			3	/* old fashioned */
#define	INTC_VBLANK_E		3
#define INTC_VIF0  			4
#define INTC_VIF1  			5
#define INTC_VU0  			6
#define INTC_VU1  			7
#define INTC_IPU  			8
#define INTC_TIM0  			9
#define INTC_TIM1  			10

#define	INTC_STAT			0x1000F000	//Interrupt status
#define	INTC_MASK			0x1000F010  //Interrupt mask

extern char * debugIntrptCause[ 11 ];

void BiosUpdate( void );


/********************************************************************************************
							Timer DECLARATIONS
*********************************************************************************************/
/*
Freq CPU = 300 Mhz
436 MIPS ( Million of Instruction per Second ) ( 436 000 000 )

Assuming :

*/


#define PS2_CLOCK 314572800		// 300 * 1024 * 1024 


#define	PS2_TIMER_0		0
#define	PS2_TIMER_1		1
#define	PS2_TIMER_2		2
#define	PS2_TIMER_3		3
#define	NSX2_TIMER		4


void TimerInit( _u8 TimerID );

void TimerUpdate( void );



/********************************************************************************************
							SIF RPC DECLARATIONS
*********************************************************************************************/

#include "SIF_RPC.h"

#define	SIF_PLUGGIN_MAX_BIND	32

#define		BIND_RPC_NPM_USER	0
#define		BIND_CDVD_IRX		1
#define		BIND_SJPCM_IRX		2	
#define		BIND_AMIGAMOD_IRX	3
	
#define		BIND_0x80000001		4
#define		BIND_0x80000003		5
#define		BIND_0x80000400		6
#define		BIND_0x80000592		7
#define		BIND_0x80000593		8
#define		BIND_0x80000595		9
#define		BIND_0x80000596		10
#define		BIND_0x80000597		11
#define		BIND_0x8000059a		12
#define		BIND_0x80000601		13
		
#define		BIND_0x80000006		14
#define		BIND_0x8000010f		15
#define		BIND_0x8000011f		16
#define		BIND_0x80000100		17
#define		BIND_0x80000101		18


#define PAD_BIND_RPC_ID1 	0x8000010f
#define PAD_BIND_RPC_ID2 	0x8000011f


#define AMIGAMOD_IRX  		0xbadca11
#define	SJPCM_IRX			0xB0110C5
#define RPC_NPM_USER 		0x014d704e


void RPC_8000010f_PadMan1	( _u32 fRPCcommand, _u64 fBinaryInAddr, _u64 fBinaryOutAddr ) ;




void PadAndKeyUpdate( void ) ;






#endif
