/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>

#include "nSX2.h"
 
#include "r5900_main.h"
#include "r5900_main_table.h"

#include "PS2Edefs.h"



/*****************************************************************************************************************
									COP1 INSTRUCTION SET
*****************************************************************************************************************/

void    COP1()		{	cop1_opc[( _OPCODE_ & 0x3E00000)>>21 ](); }

void	BC1()		{ 	cop1_bc1_opc[( _OPCODE_ >>16)& 0x3 ](); }

void	COP1_S()	{ 	cop1_s_opc[ _OPCODE_ & 0x3F ](); };

void	COP1_W()	{ 	cop1_W_L_opc[ _OPCODE_ & 0x3F ](); }

/*
*/
void	ABS_S( ) {	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: abs_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );
          
    *MipsCpu.FPR1_p_float[CURRENT_FD] = fabs( (double)(*MipsCpu.FPR1_p_float[CURRENT_FS]) );
                
	INCREMENT_PC();
} 	



/*
Floating Point Add
To add floating point values
*/
void	ADD_S()		{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: add_s %s,%s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);
           
    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) + (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
                
	INCREMENT_PC();
} 

 		

/*
*/
void 	ADDA_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 


/*
Branch on FP False
To branch according to the result of the immediately preceding floating-point comparaison operation.
*/
void	BC1F()		{ 	
	_u64	target_branch;
	_u32	condition;

	_u64 off;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1f %lx,%4llx", (long)MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;

	if ( condition == 0 ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
		
	}
	else INCREMENT_PC(); 
} 


/*
*/

void	BC1FL()		{	
	_u64	target_branch;
	_u32	condition;
	_u64 off;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1fl %lx,%4llx", (long)MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;

	if ( condition == 0 ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
		
	}
	else MipsCpu.PC = MipsCpu.PC + 8; 
} 


/*
Branch on FP True
To test an FP condition code and do a PC-relative conditional branch
*/
void	BC1T()		{ 	
	_u64	target_branch;
	_u32	condition;

	_u64 off;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1t %lx,%4llx", (long)MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;

	if ( condition == CP1_FCSR_C_MASK ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
} 




/*
Branch on FP True Likely
To test an FP condition code and do a PC-relative conditional branch; execute the instruction in the delay slot only if
the branch is taken.
*/
void	BC1TL()		{ 	
	_u64	target_branch;
	_u32	condition;

	_u64 off;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bc1tl %lx,%4llx", (long)MipsCpu.PC, CURRENT_CCB,target_branch );

	condition = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_MASK;
	
	if ( condition == CP1_FCSR_C_MASK ) {
		
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;
		
		INCREMENT_PC();
		MipsUpdateCpu();
		
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8; 
}



/*
Floating Point Compare
To compare single-precision floating-point values and record the result in flag C
*/
void	C_EQ_S()	{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_eq_s %lx,%s,%s", (long)MipsCpu.PC, CURRENT_CC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FT] );


	if ( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) == (float) (*MipsCpu.FPR1_p_float[CURRENT_FT])  ) 
		MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | CP1_FCSR_C_MASK;
	else  MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_NOT_MASK;
 
  	INCREMENT_PC();
}	



void	C_F_S()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 	

/*
*/
void	C_OLE_S()	{	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8lx: c_ole_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FT] );

	if ( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) <= (float)(*MipsCpu.FPR1_p_float[CURRENT_FT])  )
        MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | CP1_FCSR_C_MASK;
	else MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_NOT_MASK;

	INCREMENT_PC();
} 


/*
Floating Point Compare
To compare single-precision floating-point values and record the result in flag C
*/
void	C_OLT_S()	{ 	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: c_olt_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FT] );

	if ( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) < (float)(*MipsCpu.FPR1_p_float[CURRENT_FT])  )
        MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] | CP1_FCSR_C_MASK;
	else MipsCpu.CP1[CP1_FCSR] = MipsCpu.CP1[CP1_FCSR] & CP1_FCSR_C_NOT_MASK;

	INCREMENT_PC();
}	

	

void	CFC1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 		

void	CTC1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 	

/*
Fixed-point Convert to Single Floating Point
To convert a 32 bit signed integer to a single-precision floating-point value
*/
void	CVTS_W()	{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FS],debug_r5900_cop1_reg[CURRENT_FD] );

	*MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((_s32)MipsCpu.FPR1[CURRENT_FS].lo);
 
  	INCREMENT_PC();
}



/*
Floating Point Convert to Word Fixed Point
To convert a 32 bit signed integer to a single-precision fixed-point value
MIPS32
*/
void	CVT_W_S()	{ 
    _s16    f_exp;
	
    if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: cvt_w_s	%s, %s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );
       
    f_exp = (_u16)( (MipsCpu.FPR1[CURRENT_FS].lo >> 23) & 0xFF) ;
     
    if ( (_u16) f_exp <= 0x9D ) {
      MipsCpu.FPR1[CURRENT_FD].lo =(_s32)((float)*MipsCpu.FPR1_p_float[CURRENT_FS]);
    }
    else {
        if ( (MipsCpu.FPR1[CURRENT_FS].lo & SIGN_BIT_MASK ) == 0 ) MipsCpu.FPR1[CURRENT_FD].lo = 0x7FFFFFFF;
        else MipsCpu.FPR1[CURRENT_FD].lo = 0x80000000;
   }

	sign_extended( MipsCpu.FPR1[CURRENT_FD].lo );


   INCREMENT_PC();
}	

/*
Floating Point Divide
To divide FP values
MIPS32
*/
void	DIV_S()		{ 
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: div_s %s,%s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) == 0.0 )  ) {
        
        if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) > 0.0 ) *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MAX;
        else *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MIN;
    }

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) == 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) == 0.0 )  ) {
        *MipsCpu.FPR1_p_float[CURRENT_FD] = FLT_MAX;
    }

	if ( ( (*MipsCpu.FPR1_p_float[CURRENT_FS]) != 0.0 ) && ( (*MipsCpu.FPR1_p_float[CURRENT_FT]) != 0.0 )  ) {
	
   		*MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) / (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
   
    } 
	INCREMENT_PC();
} 	



/*
Load Word to Floating Point LWC1
To load a word from memory to an FPR
MIPS32
*/
void    LWC1()		{		
	_u128	temp;
	_u64 off;
	
	off = Extractoffset();
	sign_extendedh(off);
	

	if ( debug_op == 1 ) 		{
		sprintf(debugTempText,"%8.8llx: lwc1 %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
		//sprintf(debugTempText,"%8.8llx: %s = %8.8llx ", MipsCpu.PC,debug_r5900_cop1_reg[CURRENT_FT],MipsCpu.FPR1[CURRENT_FT].lo);
	}
	
	MipsMemoryRead ( &temp , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

	MipsCpu.FPR1[CURRENT_FT].lo = (_u32)temp.lo ;
	
    INCREMENT_PC();
}




void 	MADD_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void 	MADDA_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 


void 	MAX_S()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}	


/*
Move Word From Floating Point
To copy a word from an FPU (CP1) general register to a GPR
MIPS32
*/
void	MFC1()		{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mfc1 %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_cop1_reg[CURRENT_FS] );

  	MipsCpu.GPR[CURRENT_RT].lo = (_u32)MipsCpu.FPR1[CURRENT_FS].lo;
	sign_extended(MipsCpu.GPR[CURRENT_RT].lo);
 
    INCREMENT_PC();
}




void	MIN_S()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 


/*
Floating Point Move
To move an FP value between FPRs
MIPS32
*/
void	MOV_S()		{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );

    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float) (*MipsCpu.FPR1_p_float[CURRENT_FS]);

	INCREMENT_PC();
} 



void 	MSUB_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void 	MSUBA_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 


/*
Move Word to Floating Point
To copy a word from a GPR to an FPU (CP1) general register
MIPS32
*/
void	MTC1()		{ 
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mtc1 %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_cop1_reg[CURRENT_FS] );
	
	MipsCpu.FPR1[CURRENT_FS].lo = (_u32)( MipsCpu.GPR[CURRENT_RT].lo  );						//Copy the val to the reg
	INCREMENT_PC();	
}	



/*
Floating Point Multiply
To multiply FP values
MIPS32
*/
void	MUL_S()		{ 
 	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mul_s %s,%s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);

	*MipsCpu.FPR1_p_float[CURRENT_FD] =(float)( (float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) * (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));
   
 	INCREMENT_PC();
} 



void	MULA_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 
 

/*
Floating Point Negate
To negate an FP value
MIPS32
*/
void	NEG_S()		{ 	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: mov_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] );

    *MipsCpu.FPR1_p_float[CURRENT_FD] = (float)(-(*MipsCpu.FPR1_p_float[CURRENT_FS]));
    
	INCREMENT_PC();
}



void	RSQRT_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}



/*
*/
void	SQRT_S()	{	
	
	float fresult;

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: sqrt_s %s,%s", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FT],debug_r5900_cop1_reg[CURRENT_FD] );
	
	if ( *MipsCpu.FPR1_p_float[CURRENT_FT] >= 0 )
		fresult = sqrt( *MipsCpu.FPR1_p_float[CURRENT_FT] );
	else 
		fresult = sqrt( -(*MipsCpu.FPR1_p_float[CURRENT_FT]) );

	*MipsCpu.FPR1_p_float[CURRENT_FD] = fresult;
	
	
	INCREMENT_PC();
}		



void	SUB_S()		{ 

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: sub_s %s,%s,%s\n", (long)MipsCpu.PC, debug_r5900_cop1_reg[CURRENT_FD],debug_r5900_cop1_reg[CURRENT_FS] ,debug_r5900_cop1_reg[CURRENT_FT]);

	*MipsCpu.FPR1_p_float[CURRENT_FD] = (float)((float)(*MipsCpu.FPR1_p_float[CURRENT_FS]) - (float)(*MipsCpu.FPR1_p_float[CURRENT_FT]));

	INCREMENT_PC();
} 		

		


void 	SUBA_S()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 


/*
Store Word from Floating Point
To store a word from an FPR to memory
MIPS32
*/
void    SWC1()		{

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdwc1 fp%ld,%4llx(%s)", (long)MipsCpu.PC, CURRENT_FT,CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	MipsMemoryWrite ( MipsCpu.FPR1[CURRENT_FT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

    INCREMENT_PC();
}





