/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>

#include "nSX2.h"
 
#include "r5900_main.h"
#include "r5900_main_table.h"

#include "PS2Edefs.h"
 


 
/*
Print the the current opcode
*/
void PrintDebugOpcode() {
	
	char 		tempText[100];		//for displaying text to the widget
/*
	sprintf(tempText,"%d:",MipsCpu.IndexListPrefetchedOpcode);
	PrintToEmuOutput(tempText);
*/	

	if ( MipsCpu.opcode == 0 ) return;

	sprintf(tempText,"%8.8lx:(opcode:%8.8lx):", (long)MipsCpu.PC,MipsCpu.opcode);
	PrintToEmuOutput(tempText);
	
	switch ( ((MipsCpu.opcode >> 26) & 0x3F) ) {
	
		case 0 	: 	sprintf(tempText,"r5900:dbg_opcode_special: %s \n", debug_special_opc[(MipsCpu.opcode  & 0x3F)]); break;
		case 1	: 	sprintf(tempText,"r5900:dbg_opcode_regimm: %s \n", debug_regimm_opc[(MipsCpu.opcode  & 0x1F0000)>>16]); break;
		case 16 : 	sprintf(tempText,"r5900:dbg_opcode_cop0: %s \n", debug_cop0_opc[(MipsCpu.opcode&0x3E00000)>>21]);
				  	sprintf(tempText,"r5900:dbg_opcode_cop0c0: %s \n", debug_cop0c0_opc[(MipsCpu.opcode&0x3F)]);
					break;
		case 17 : {
					switch( (MipsCpu.opcode&0x3E00000)>>21) {
						
						case 8:  sprintf(tempText,"r5900:debug_cop1_BC1_opc: %s \n",	debug_cop1_bc1_opc[(MipsCpu.opcode>>16)&0x3] );break;					
						case 16: sprintf(tempText,"r5900:debug_cop1_S_opc: %s \n", 	debug_cop1_s_opc[MipsCpu.opcode&0x3F] 	);break;					
						case 20: sprintf(tempText,"r5900:debug_cop1_W_opc: %s \n", 	debug_cop1_W_L_opc[MipsCpu.opcode&0x3F] );break;					

						default: sprintf(tempText,"r5900:dbg_opcode_cop1: %s \n", debug_cop1_opc[(MipsCpu.opcode&0x3E00000)>>21]);break;					
					}
					
					break;
				}	
		case 18	: {
					if ( ((MipsCpu.opcode&0x3E00000)>>21) > 15 ) {
						

						switch ( MipsCpu.opcode&0x3F ) {

							case 60 : sprintf(tempText,"r5900:debug_cop2_V_t60_opc: %s \n", debug_cop2_V_t60_opc[(MipsCpu.opcode & 0x7C0)>>6]);break;
							case 61 : sprintf(tempText,"r5900:debug_cop2_V_t61_opc: %s \n", debug_cop2_V_t61_opc[(MipsCpu.opcode & 0x7C0)>>6]);break;
							case 62 : sprintf(tempText,"r5900:debug_cop2_V_t62_opc: %s \n", debug_cop2_V_t62_opc[(MipsCpu.opcode & 0x7C0)>>6]);break;
							case 63: sprintf(tempText,"r5900:debug_cop2_V_t63_opc: %s \n", debug_cop2_V_t63_opc[(MipsCpu.opcode & 0x7C0)>>6]);break;
							default : sprintf(tempText,"r5900:debug_pcop2_Vopc: %s \n", debug_pcop2_V_opc[MipsCpu.opcode&0x3F]);break;
						}


					} else { 

						sprintf(tempText,"r5900:debug_pcop2_opc: %s \n", debug_pcop2_opc[(MipsCpu.opcode&0x3E00000)>>21]);
					}

				}
				break;

		case 28 : sprintf(tempText,"r5900:dbg_opcode_special2: %s \n", debug_special2_opc[(MipsCpu.opcode  & 0x3F)] );break;
	
		default : sprintf(tempText,"r5900:dbg_opcode: %s \n", debug_normal_opc[((MipsCpu.opcode >> 26) & 0x3F)]); break;
	
	}
	PrintToEmuOutput(tempText);
}
 

/*
Main CPU function
( See r5900_main.h for #define definitions )
*/
void MipsUpdateCpu ( ) {

	/*Update the Number of Instructions*/
	MipsCpu.Nb_instructions++;		
	
	/* read the Memory at PC and Set MipsCpu.opcode */
	MipsFillOpcode(MipsCpu.PC);

	if ( _OPCODE_ != 0 ) normal_opc[((_OPCODE_ >> 26) & 0x3F)]();	//Execute the current opcode
	else NOP();

	if ( debug_op == 1 ) {
		PrintToOpcodeOutput ( debugTempText );
		PrintToRegistersOutput( NULL );
		PrintToCOP1RegistersOutput (  NULL );
		PrintExtraStuff();
	}
	
}


/* Test if a value is SignExtended : NEED TO BE CHECKED*/
_u8 IsSignExtended( _u64 fval ) {

    _u64    temp;
/*   
    temp = fval & 0xFFFFFFFF00000000;
    if (    ( temp == 0 ) || ( temp == 0xFFFFFFFF00000000 ) ) return TRUE;
    else return FALSE;        
*/
	return TRUE;
   
}

/* Test if a value is Overflow : NEED TO BE CHECKED*/
_u8 IsNot32bitIntegerOverflow ( _u64  fval) {

    _u32    temp;

/*	
	if ( (_u64)fval < 0xFFFFFFFF ) return TRUE;
	else {
		if ( (_u64)fval & 0xFFFFFFFF00000000 ) return TRUE;
		else return FALSE;
		}
*/
	return TRUE;

}


/*
Not sure
*/
_u8 IsNot64bitIntegerOverflow ( _u64  fval) {
/*
    _u64    temp;
   
    temp = ( (~fval) + 1 );

    if ( temp == 0 ) return FALSE;
    else return TRUE;
*/
	return TRUE;
        
}

/*
Not Implemented function
*/
void    NI(){	PrintDebugOpcode();MipsCpu.halt = 1;}


/*
N OPeration function
*/
void    NOP() 		{ 	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: nop", (long)MipsCpu.PC);
	INCREMENT_PC(); 
}
  

/*****************************************************************************************************************
									CPU INSTRUCTION SET
*****************************************************************************************************************/

void 	SPECIAL()	{	special_opc[(_OPCODE_  & 0x3F)]();	}

void 	REGIMM()	{	regimm_opc[(_OPCODE_  & 0x1F0000)>>16]();		}


/*
Add Word
To add 32-bit integers. If an overflow occurs, then trap.
*/
void   ADD     () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: add	%s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS] );

	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) {
		
		temp =(_u64)(_u64)MipsCpu.GPR[CURRENT_RS].lo + (_u64)MipsCpu.GPR[CURRENT_RT].lo;
		
		if ( IsNot32bitIntegerOverflow ( temp ) ) {
		   sign_extended(temp);
		   MipsCpu.GPR[CURRENT_RD].lo = (_u64)temp;
		}
		else {
			//SignalExeption(IntegerOverflow)
		}
	}
	
	INCREMENT_PC();
}



/*
Add Immediate Word
To add 32-bit integers. If an overflow occurs, then trap.
*/
void    ADDI   () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: addi %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) ) {

		temp = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo + (_s16)CURRENT_IMM);
		
		if ( IsNot32bitIntegerOverflow ( temp ) ) {
				sign_extended(temp);
				MipsCpu.GPR[CURRENT_RT].lo = (_u64)temp;
				
				
			
		} else {	   
			//SignalExeption(IntegerOverflow)
		}
	}
	INCREMENT_PC();
}




/*
Add Immediate Unsigned Word
To add a constant to a 32-bit integer
*/
void    ADDIU () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: addiu %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) ) {

	 	MipsCpu.GPR[CURRENT_RT].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo + (_s16)CURRENT_IMM);
		sign_extended(MipsCpu.GPR[CURRENT_RT].lo);
	    
    }
    
    INCREMENT_PC();
}




/*
Add Unsigned Word
To add 32-bit integers
*/
void   ADDU    () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: addu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS] );
	
    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) {
		
		MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo + (_u64)MipsCpu.GPR[CURRENT_RT].lo);
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}



/*
And
To do a bitwise logical AND
*/
void   AND () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: and	%s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	MipsCpu.GPR[CURRENT_RD].lo =(_u64)( (_u64)MipsCpu.GPR[CURRENT_RS].lo & (_u64)MipsCpu.GPR[CURRENT_RT].lo);
	
	INCREMENT_PC();
}


/*
And Immediate
To do a bitwise logical AND with a constant
*/
void    ANDI() {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: andi %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
	MipsCpu.GPR[CURRENT_RT].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo & (_u64)CURRENT_IMM);
	
	INCREMENT_PC();
}



/*
Branch on Equal
To compare GPRs then do a PC-relative conditional branch
*/
void     BEQ () {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: beq %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( (_u64)MipsCpu.GPR[CURRENT_RS].lo == (_u64)MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
	
		CalculTargetBranch;	
	
        INCREMENT_PC();
		MipsUpdateCpu();
		
        MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
} 


/*
Branch on Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void     BEQL() {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: beql %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( (_u64)MipsCpu.GPR[CURRENT_RS].lo == (_u64)MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}




/*
Branch on Greater Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
*/
void      BGEZ() {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgez %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );

	if ( (_s64)(MipsCpu.GPR[CURRENT_RS].lo) >= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}


void      BGEZAL () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void      BGEZALL() {	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void      BGEZL  () {		
	_u64	target_branch;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgezl %s,%4llx", (long)MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );

	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo >= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


/*
Branch on Greater Than Zero
To test a GPR then do a PC-relative conditional branch
*/
void     BGTZ () {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgtz %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );

	if ( (_s64)( MipsCpu.GPR[CURRENT_RS].lo ) > 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );


		CalculTargetBranch;


		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC(); 
}




/*
Branch on Greater Than Zero Likely
To do a PC-relative branch according to the values of a GPR. Executes the instruction
in the branch delay sloat only if the branch is taken.

*/
void    BGTZL() {		
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bgtz %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );

	if ( (_s64)( MipsCpu.GPR[CURRENT_RS].lo ) > 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8; 
}



/*
Branch on Less Than or Equal to Zero
To test a GPR then do a PC-relative conditional branch
*/
void     BLEZ () {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blez %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );
	
	if ( (_s64)(MipsCpu.GPR[CURRENT_RS].lo) <= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}



/*
Branch on Less Than or Equal to Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void    BLEZL() {		
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: blezl %s,%4llx", (long)MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );
	
	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo <= 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}




/*
Branch on Less Than Zero Likely
To test a GPR then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void	BLTZ() {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltz %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS], target_branch );
	
	if ((_s64)( MipsCpu.GPR[CURRENT_RS].lo) < 0 ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}


void      BLTZALL() {	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void      BLTZAL () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}


/*
Branch on Less Than Zero Likely
*/
void      BLTZL  () {		
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bltzl %s,%4llx", (long)MipsCpu.PC,debug_r5900_reg[CURRENT_RS],target_branch );

	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo < 0 ) {

		CalculTargetBranch;

		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );
		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}






/*
Branch on Not Equal
To compare GPRs then do a PC-relative conditional branch
*/
void     BNE () {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8lx: bne %s, %s, %4llx",(long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( (_u64)MipsCpu.GPR[CURRENT_RS].lo != (_u64)MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else INCREMENT_PC();
}




/*
Branch on Not Equal Likely
To compare GPRs then do a PC-relative conditional branch; execute the delay slot only if the branch is taken.
*/
void     BNEL() {
	_u64	target_branch;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: bnel %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT],target_branch );
	
	if ( (_u64)MipsCpu.GPR[CURRENT_RS].lo != (_u64)MipsCpu.GPR[CURRENT_RT].lo ) {
		if ( debug_op == 1 ) PrintToOpcodeOutput ( debugTempText );

		CalculTargetBranch;

		INCREMENT_PC();
		MipsUpdateCpu();
		MipsCpu.PC = target_branch;
	}
	else MipsCpu.PC = MipsCpu.PC + 8;
}


 
/*
Breakpoint
To cause a Breakpoint exception
*/
void	BREAK()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void	DADD()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void	DADDI()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}



/*
Doubleword Add Immediate Unsigned
To add a constant to a 64-bit integer
*/
void    DADDIU    () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddiu %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

	MipsCpu.GPR[CURRENT_RT].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo + (_s16)CURRENT_IMM);

	INCREMENT_PC();
}




/*
Doubleword Add Unsigned
To add 64-bit integers
*/
void   DADDU  () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: daddu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo + (_u64)MipsCpu.GPR[CURRENT_RT].lo);
	
	INCREMENT_PC();
}


/*
Divide Word
To divide a 32-bit signed integers
*/
void   DIV  () {		
	
	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: div %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) && ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   

       MipsCpu.GPR[CPU_REG_LO].lo = (_s32)MipsCpu.GPR[CURRENT_RS].lo / (_s32)MipsCpu.GPR[CURRENT_RT].lo;
       MipsCpu.GPR[CPU_REG_HI].lo = (_s32)MipsCpu.GPR[CURRENT_RS].lo % (_s32)MipsCpu.GPR[CURRENT_RT].lo;
 		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);
		
	}
	
	INCREMENT_PC();
}


/*
Divide Unsigned Word
To divide a 32-bit unsigned integers
*/
void   DIVU () {

	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: divu %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) && ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   

       MipsCpu.GPR[CPU_REG_LO].lo = (_u32)MipsCpu.GPR[CURRENT_RS].lo / (_u32)MipsCpu.GPR[CURRENT_RT].lo;
       MipsCpu.GPR[CPU_REG_HI].lo = (_u32)MipsCpu.GPR[CURRENT_RS].lo % (_u32)MipsCpu.GPR[CURRENT_RT].lo;

		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].lo);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].lo);
    
    }
	
	INCREMENT_PC();
}



/*
Doubleword Shift Left Logical Variable
To execute a left-shift of a doubleword by a variable number of bits
*/
void   DSLL     () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll %s,%s,%4lx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );

	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo << CURRENT_SA) ;
	
	INCREMENT_PC();
}



/*
Doubleword Shift Left Logical Plus 32
To execute a left-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSLL32  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsll32 %s,%s,%4lx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo <<  (CURRENT_SA + 32 )) ;

	INCREMENT_PC();
}


/*
Doubleword Shift Left Logical Variable
To shift Left a doubleword. The shift amount is a variable ( specified by a GPR )

*/
void   DSLLV () {	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsllv %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS] );

	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo <<  ( MipsCpu.GPR[CURRENT_RS].lo & 0x3F));
 
 
 	INCREMENT_PC();
}




/*

*/
void   DSRA  () {
	_u64 temp;
	_u8 boucle;
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsra %s,%s,%ld", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	
		if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK_64) == SIGN_BIT_MASK_64) {
			
			temp = (_u64)MipsCpu.GPR[CURRENT_RT].lo ;
			for ( boucle = 0 ; boucle < CURRENT_SA ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK_64 ;
			MipsCpu.GPR[CURRENT_RD].lo = (_u64)temp;
			
		} else 	MipsCpu.GPR[CURRENT_RD].lo =  (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo >>  CURRENT_SA ) ; 
	
      		
	INCREMENT_PC();
}






/*
Doubleword Shift Right Arithmetic Plus 32
To execute an arithmetic right-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSRA32() {
	_u8	 boucle;	
	_u64 temp;


	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsra32 %s,%s,%4lx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK_64) == SIGN_BIT_MASK_64 ) { 
		temp = (_u64)MipsCpu.GPR[CURRENT_RT].lo;
		for ( boucle = 0 ; boucle < (CURRENT_SA+32)  ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK_64 ;
	
	} else 	temp = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo >>  (CURRENT_SA+32))  ;

	MipsCpu.GPR[CURRENT_RD].lo = temp;
	INCREMENT_PC();
}


void   DSRAV () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}

/*
Doubleword Shift Right Logical DSRL
To execute a logical right-shift of a doubleword by a fixed amount 0 to 31 bits
*/
void   DSRL  () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl %s,%s,%4lx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo >> CURRENT_SA ) ;
	
	INCREMENT_PC();
}




/*
Doubleword Shift Right Logical Plus 32
To execute a logical right-shift of a doubleword by a fixed amount 32 to 63 bits
*/
void   DSRL32   () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsrl32 %s,%s,%4lx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT],CURRENT_SA );

	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo >>  (CURRENT_SA+32) )  ;
	
	INCREMENT_PC();
}


/*

*/

void   DSRLV () {	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srlv %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
		
	MipsCpu.GPR[CURRENT_RD].lo =  (_u64)((_u64)MipsCpu.GPR[CURRENT_RT].lo   >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x3F));	
		
	INCREMENT_PC();
}



/*
*/

void   DSUB  () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}

/*
Doubleword Subtract Unsigned
To subtract 64-bit integers
*/
void   DSUBU() {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: dsubu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo - (_u64)MipsCpu.GPR[CURRENT_RT].lo);
	
	INCREMENT_PC();
}




/*
Jump
To branch within the current 256 MB-aligned region
*/
void J() {
	_u64 	target_branch;
	
	target_branch = (_u32)(CURRENT_INSTR_INDEX << 2 ) ;
	
	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: j %8llx", (long)MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  target_branch;
}



/*
Jump and Link
To execute a procedure call within the current 256 MB-aligned region
*/
void     JAL  () {
	_u64 	target_branch;
	
	MipsCpu.GPR[ CPU_REG_RA ].lo =  MipsCpu.PC + 8;
	target_branch = CURRENT_INSTR_INDEX  << 2;
	
	if ( debug_op == 1 ) 	{
		sprintf(debugTempText,"%8.8llx: jal %8llx", (long)MipsCpu.PC, target_branch );
		PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  target_branch;
}



/*
Jump and Link Register
To execute a procedure call to an instruction address in a register
*/
void   JALR () {
	_u64	branch_target;
	
	branch_target = MipsCpu.GPR[CURRENT_RS].lo;
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.PC + 8;
	
	if ( debug_op == 1 ) {
	    sprintf(debugTempText,"%8.8llx: jalr %s,%s with %s=%8llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RS], MipsCpu.GPR[CURRENT_RS].lo );
	    PrintToOpcodeOutput ( debugTempText );
	}
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  branch_target;
}



/*
Jump Register
To execute a branch to an instruction address in a register
*/
void   JR   () {
	_u64	branch_target;
	
	if ( debug_op == 1 ) {
	     sprintf(debugTempText,"%8.8llx: jr %s with %s=%8llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RS] , MipsCpu.GPR[CURRENT_RS].lo );
	     PrintToOpcodeOutput ( debugTempText );
	}
	
	branch_target = MipsCpu.GPR[CURRENT_RS].lo;
	
	INCREMENT_PC();
	MipsUpdateCpu();
	
	MipsCpu.PC =  branch_target;
}



/*
Load Byte
To load a byte from memory as a signed value
*/
void    LB     () {
	_u64 off;

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lb %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	
	off = Extractoffset();
	sign_extendedh(off);

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

	sign_extendedb( MipsCpu.GPR[CURRENT_RT].lo );
	
	INCREMENT_PC();
}



/*
Load Byte Unsigned
To load a byte from memory as an unsigned value
*/
void    LBU () {
	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lbu %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

    INCREMENT_PC();
}



/*
Load Doubleword
To load a doubleword from memory
*/
void    LD   () {

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: ld %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	
	INCREMENT_PC();
}




/*
Load Doubleword Left
To load the most-significant part of a doubleword from an unaligned memory address
*/
void    LDL  () {
    _u128   temp128;	
    _u64    ReadAdress;

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldl %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
	//sign_extendedh(offset);
 
	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFFFFFFFFFF) | (temp128.lo << 56);
		break;

		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFFFFFFFFFF) | (temp128.lo << 48);
		break;

		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FFFFFFFFFF ) | (temp128.lo << 40);
		break;
	
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000FFFFFFFF ) | (temp128.lo << 32);
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000000000FFFFFF) | (temp128.lo << 24);
		break;

		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000000000FFFF ) | (temp128.lo << 16);
		break;
	
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000000000FF ) | (temp128.lo << 8);
		break;
	
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo ;
		break;
		
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFFFFFFFFFF) | (temp128.lo << 56);
		break;
	
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFFFFFFFFFF) | (temp128.lo << 48);
		break;
	
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FFFFFFFFFF ) | (temp128.lo << 40);
		break;
	
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000FFFFFFFF ) | (temp128.lo << 32);
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000000000FFFFFF) | (temp128.lo << 24);
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000000000FFFF ) | (temp128.lo << 16);
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00000000000000FF ) | (temp128.lo << 8);
		break;

		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo ;
		break;
     }

	INCREMENT_PC();
}




/*
Load Doubleword Right
To load the least-significant part of a doubleword from an unaligned memory address
*/
void    LDR   () {
	_u128  temp128;
    _u64    ReadAdress;

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);



	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldr %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo;
		break;
		
		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFF00000000000000) | (temp128.lo >> 8);
		break;
		
		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFF000000000000) | (temp128.lo >> 16);
		break;
		
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFF0000000000) | (temp128.lo >> 24);
		break;
		
		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF00000000) | (temp128.lo >> 32);
		break;
		
		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | (temp128.lo >> 40);
		break;
		
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | (temp128.lo >> 48);
		break;
		
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | (temp128.lo >> 56);
		break;
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo;
		break;
		
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFF00000000000000) | (temp128.lo >> 8);
		break;
		
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFF000000000000) | (temp128.lo >> 16);
		break;
		
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFF0000000000) | (temp128.lo >> 24);
		break;
		
		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF00000000) | (temp128.lo >> 32);
		break;
		
		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | (temp128.lo >> 40);
		break;
		
		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | (temp128.lo >> 48);
		break;
		
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | (temp128.lo >> 56);
		break;	
	}


	INCREMENT_PC();
}



/*
Load Halfword
To load a halfword from memory as a signed value
*/
void    LH    () {

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lh %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	sign_extendedh( MipsCpu.GPR[CURRENT_RT].lo );
	
	INCREMENT_PC();
}


/*
Load Halfword Unsigned
To load a halfword from memory as an unsigned value
*/
void    LHU () {

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lhu %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	
	INCREMENT_PC();
}



/*
Load Upper Immediate
To load a constant into the upper half of a word
*/
void    LUI  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: lui %s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_IMM );
  
    MipsCpu.GPR[CURRENT_RT].lo = CURRENT_IMM << 16;
 	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );
 	
    INCREMENT_PC();
}



/*
Load Word
To load a word from memory as a signed value
*/
void    LW   () {

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lw %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	sign_extended( MipsCpu.GPR[CURRENT_RT].lo );

    INCREMENT_PC();


}




/*
Load Word Left
To load the most-significant part of a word as a signed value from an unaligned memory address
*/

void    LWL  () {
    _u128   temp128;	
    _u64    ReadAdress;

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);
	
	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldl %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	
 	 
	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo << 24) & 0xFF000000);
		break;

		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo << 16) & 0xFFFF0000);
		break;

		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo << 8) & 0xFFFFFF00);
		break;
	
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo >> 8 ) & 0xFF00000000);
		break;

		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo >> 16) & 0xFFFF0000);
		break;

		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo >> 24) & 0xFFFFFF00);
		break;
	
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (temp128.lo >> 32) & 0xFFFFFFFF;
		break;

		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo << 24) & 0xFF000000);
		break;

		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo << 16) & 0xFFFF0000);
		break;

		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo << 8) & 0xFFFFFF00);
		break;
	
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x00FFFFFF) | ((temp128.lo >> 8 ) & 0xFF00000000);
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x0000FFFF) | ((temp128.lo >> 16) & 0xFFFF0000);
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0x000000FF ) | ((temp128.lo >> 24) & 0xFFFFFF00);
		break;
	
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (temp128.lo >> 32) & 0xFFFFFFFF;
		break;




     }

	INCREMENT_PC();
}





/*
Load Word Right
To load the least-significant part of a word from an unaligned memory address as a signed value
*/

void    LWR   () {
	_u128  temp128;
    _u64    ReadAdress;



	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 )		sprintf(debugTempText,"%8.8llx: ldr %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + CURRENT_OFFSET ) & 0xF)
	{
		case 0:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
			//sign_extended( MipsCpu.GPR[CURRENT_RT].lo );
		break;
		
		case 1:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 2:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 3:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		case 4:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
			//sign_extended( MipsCpu.GPR[CURRENT_RT].lo );			
		break;
		
		case 5:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 6:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 7:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;
		
		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
			//sign_extended( MipsCpu.GPR[CURRENT_RT].lo );			
		break;
		
		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = temp128.lo & 0xFFFFFFFF;
			//sign_extended( MipsCpu.GPR[CURRENT_RT].lo );			
		break;
		
		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFF000000) | ((temp128.lo >> 8 )& 0xFFFFFF);
		break;
		
		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFF0000) | ((temp128.lo >> 16) & 0xFFFF);
		break;
		
		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			MipsCpu.GPR[CURRENT_RT].lo = (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFFFFFFFF00) | ((temp128.lo >> 24) & 0xFF);
		break;

		

	}


	INCREMENT_PC();
}



/*
Load Word Unsigned
To load a word from memory as an unsigned value
*/
void    LWU   () {
	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lwu %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	
	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

    INCREMENT_PC();
}


/*
Move From HI Register
To copy the special purpose HI register to a GPR
*/
void   MFHI   () {

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mhi %s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)MipsCpu.GPR[CPU_REG_HI].lo;

    INCREMENT_PC();
}



/*
Move From LO Register
To copy the special purpose LO register to a GPR
*/
void   MFLO  () {		

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mflo %s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)MipsCpu.GPR[CPU_REG_LO].lo;

    INCREMENT_PC();
}



/*
Move Conditional on Not Zero
To conditionally move a GPR after testing a GPR value
*/
void   MOVN() {

	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movn %s,%s,%s ", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RT] );
	
	if(  MipsCpu.GPR[CURRENT_RT].lo != 0 ) MipsCpu.GPR[CURRENT_RD].lo = (_u64)MipsCpu.GPR[CURRENT_RS].lo ;
		
	INCREMENT_PC();
}


/*
Move Conditional on Zero
To conditionally move a GPR after testing a GPR value
*/
void   MOVZ() {		
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: movz %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS] ,debug_r5900_reg[CURRENT_RT] );
	
	if(  MipsCpu.GPR[CURRENT_RT].lo == 0 ) MipsCpu.GPR[CURRENT_RD].lo = (_u64)MipsCpu.GPR[CURRENT_RS].lo ;
		
	INCREMENT_PC();
}



void   MTHI  () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}

/*

*/
void   MTLO  () {		
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mtlo %s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS]  );
	
	MipsCpu.GPR[CPU_REG_LO].lo = (_u64)MipsCpu.GPR[CURRENT_RS].lo ;

    INCREMENT_PC();
}


/*
Multiply Word
To multiply 32-bit signed integers
*/
void   MULT     () {
	_s64 temp;

	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) {   
		
		temp = (_s64)((_s32)(MipsCpu.GPR[CURRENT_RS].lo  )) * ((_s32)( MipsCpu.GPR[CURRENT_RT].lo));
		
		MipsCpu.GPR[CPU_REG_LO].lo = (_u32)temp ;
		MipsCpu.GPR[CPU_REG_HI].lo = (_u32)((_u64)temp >> 32) ;
		if ( CURRENT_RD != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo ;

    }
	
	INCREMENT_PC();
}



void   MULTU() {		
    _u64 temp;
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: multu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) {   
		
		temp = (_u64)((_u32)(MipsCpu.GPR[CURRENT_RS].lo  )) * ((_u32)( MipsCpu.GPR[CURRENT_RT].lo ));
		
		MipsCpu.GPR[CPU_REG_LO].lo = (_u32)temp ;
		MipsCpu.GPR[CPU_REG_HI].lo = (_u32)((_u64)temp >> 32) ;
		
		
		if ( CURRENT_RD != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo ;

    }
	
	INCREMENT_PC();
}



/*
Nor
To perform a bitwise logical NOT OR

*/
void   NOR   () {	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: nor %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    MipsCpu.GPR[CURRENT_RD].lo = ~((_u64)MipsCpu.GPR[CURRENT_RS].lo | (_u64)MipsCpu.GPR[CURRENT_RT].lo);

    INCREMENT_PC();
 
}



/*
Or
To do a bitwise logical OR
*/
void   OR () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

    MipsCpu.GPR[CURRENT_RD].lo =(_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo | (_u64)MipsCpu.GPR[CURRENT_RT].lo);
 
    INCREMENT_PC();
}



/*
Or Immediate
To do a bitwise logical OR with a constant
*/
void    ORI () {
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: ori	%s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );

	MipsCpu.GPR[CURRENT_RT].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo | (_u64)CURRENT_IMM);
	
	INCREMENT_PC();
}



void    PREF () {	PrintDebugOpcode() ; MipsCpu.halt = 1;}


/*
Store Byte
To store a byte to memory
*/
void    SB     () {
	
	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sb %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );


	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_B ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	
	INCREMENT_PC();
}



/*
Store Doubleword
To store a doubleword to memory
*/
void    SD() {

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );
	

	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	
	INCREMENT_PC();
}




/*
Store Doubleword Left
To store the most-significant part of a doubleword to an unaligned memory address
*/
void    SDL  () {

    _u128   temp128;
    _u64    ReadAdress;


	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);


	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdl %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xF)
	{
		case 0:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 56)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 1:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 48)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 2:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 40)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 3:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 32)|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 4:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 24)|(temp128.lo & 0xFFFFFF0000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 5:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 16)|(temp128.lo & 0xFFFF000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 6:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 8)|(temp128.lo & 0xFF00000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 7:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 8:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 56)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 9:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 48)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 10:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 40)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 11:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 32)|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 12:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 24)|(temp128.lo & 0xFFFFFF0000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 13:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 16)|(temp128.lo & 0xFFFF000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 14:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((_u64)MipsCpu.GPR[CURRENT_RT].lo >> 8)|(temp128.lo & 0xFF00000000000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 15:
			ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
			MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
		break;
	}

	INCREMENT_PC();
}


/*
Store Doubleword Right
To store the least-significant part of a doubleword to an unaligned memory address
*/

void    SDR  () {
    _u128   temp128;
    _u64    ReadAdress;

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);



	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdr %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	//sign_extendedh(offset);

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xF)
	{
	case 0:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 1:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 8) | (temp128.lo & 0x00000000000000FF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 2:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 16) | (temp128.lo & 0x000000000000FFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 3:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 24) | (temp128.lo & 0x0000000000FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 4:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 32) | (temp128.lo & 0x00000000FFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 5:	
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 40) | (temp128.lo & 0x000000FFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 6:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 48) | (temp128.lo & 0x0000FFFFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 7:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 56) | (temp128.lo & 0x00FFFFFFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 8:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 9:	
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 8) | (temp128.lo & 0x00000000000000FF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 10:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 16) | (temp128.lo & 0x000000000000FFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	
	case 11:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 24) | (temp128.lo & 0x0000000000FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 12:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 32) | (temp128.lo & 0x00000000FFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 13:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 40) | (temp128.lo & 0x000000FFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 14:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 48) | (temp128.lo & 0x0000FFFFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;

	case 15:
		ReadAdress = ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ) + 8;
		MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		temp128.lo = (MipsCpu.GPR[CURRENT_RT].lo << 56) | (temp128.lo & 0x00FFFFFFFFFFFFFF);
		MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
	break;
	}

	INCREMENT_PC();
}





/*
Store Halfword
To store a halfword to memory
*/
void    SH    () {
	
	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sh %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	

	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_HW ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
	
	INCREMENT_PC();
}


/*
Shift Word Left Logical
To left-shift a word by a fixed number of bits
*/
void   SLL() {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sll %s,%s,%ld", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );

	MipsCpu.GPR[CURRENT_RD].lo = ((_u32)MipsCpu.GPR[CURRENT_RT].lo ) <<  CURRENT_SA;
	sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	INCREMENT_PC();
}



/*
Shift Word Left Logical Variable
To left-shift a word by a variable number of bits
*/
void   SLLV () {		
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sllv %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
 
	MipsCpu.GPR[CURRENT_RD].lo = ( (_u32)MipsCpu.GPR[CURRENT_RT].lo  ) <<  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F);	
	sign_extended(MipsCpu.GPR[CURRENT_RD].lo);

	INCREMENT_PC();
}


/*
Set on Less Than
To record the result of a less-than comparison
*/
void   SLT () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slt %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	if  ((_s64) MipsCpu.GPR[CURRENT_RS].lo < (_s64) MipsCpu.GPR[CURRENT_RT].lo ) MipsCpu.GPR[CURRENT_RD].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RD].lo = 0; 
	
	INCREMENT_PC();
}



/*
Set on Less Than Immediate
To record the result of a less-than comparison with a constant
*/
void    SLTI() {
	
	_u64 imm;
	imm = Extractimm();
	sign_extendedh(imm);


	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: slti %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	//sign_extendedh(imm);
	
	if ( (_s64)MipsCpu.GPR[CURRENT_RS].lo < (_s64) imm ) 	MipsCpu.GPR[CURRENT_RT].lo = 1;
	else MipsCpu.GPR[CURRENT_RT].lo = 0;
	
	INCREMENT_PC();
}




/*
Set on Less Than Immediate Unsigned
To record the result of an unsigned less-than comparison with a constant
*/
void    SLTIU() {

	_u64 imm;
	imm = Extractimm();
	sign_extendedh(imm);



	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltiu %s,%s,%8llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS], CURRENT_IMM);
	
    //sign_extendedh(imm);
	
	if  ( (_u64)MipsCpu.GPR[CURRENT_RS].lo < (_u64)imm ) 	MipsCpu.GPR[CURRENT_RT].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RT].lo = 0; 
	
	INCREMENT_PC();
}



/*
Set on Less Than Unsigned
To record the result of an unsigned less-than comparison
*/
void   SLTU   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sltu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RS], debug_r5900_reg[CURRENT_RT]);
	
	if  ( (_u64) MipsCpu.GPR[CURRENT_RS].lo < (_u64)MipsCpu.GPR[CURRENT_RT].lo ) 	MipsCpu.GPR[CURRENT_RD].lo = 1; 
	else 	MipsCpu.GPR[CURRENT_RD].lo = 0; 
	
	INCREMENT_PC();
}



/*
Shift Word Right Arithmetic
To execute an arithmetic right-shift of a word by a fixed number of bits
*/
void   SRA  () {
	_u64 temp;
	_u8 boucle;

	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sra %s,%s,%ld", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
	
		if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK) == SIGN_BIT_MASK) {
			
			temp = (_u32)MipsCpu.GPR[CURRENT_RT].lo ;
			for ( boucle = 0 ; boucle < CURRENT_SA ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK ;
			MipsCpu.GPR[CURRENT_RD].lo = temp;
			
		} else 	MipsCpu.GPR[CURRENT_RD].lo =  (_u64)MipsCpu.GPR[CURRENT_RT].lo >>  CURRENT_SA  ; 
	
	}
     		
	INCREMENT_PC();
}


/*
Shift Word Right Arithmetic Variable
To execute an arithmetic right-shift of a word by a variable number of bits
*/
void   SRAV () {		
	_u64 temp;
	_u8 boucle;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srav %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		if ((MipsCpu.GPR[CURRENT_RT].lo & SIGN_BIT_MASK) == SIGN_BIT_MASK) {
			
			temp = (_u32)MipsCpu.GPR[CURRENT_RT].lo ;
			for ( boucle = 0 ; boucle < ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F) ; boucle++ ) temp = ( temp >> 1 ) | SIGN_BIT_MASK ;
			MipsCpu.GPR[CURRENT_RD].lo = temp;
			
		} else 	MipsCpu.GPR[CURRENT_RD].lo =  (_u64)MipsCpu.GPR[CURRENT_RT].lo >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F)  ; 


//		MipsCpu.GPR[CURRENT_RD].lo =  (_s64)((_u32)MipsCpu.GPR[CURRENT_RT].lo >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F)) ; 
	
	}

	
	INCREMENT_PC();
}



/*
Shift Word Right Logical
To execute a logical right-shift of a word by a fixed number of bits
*/
void   SRL   () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srl %s,%s,%ld", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], CURRENT_SA );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = (_s64)(((_u32)MipsCpu.GPR[CURRENT_RT].lo)  >>  CURRENT_SA);	
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	}	
	INCREMENT_PC();
}



/*
Shift Word Right Logical Variable
To execute a logical right-shift of a word by a variable number of bits
*/
void   SRLV () {	

	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: srlv %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD], debug_r5900_reg[CURRENT_RT], debug_r5900_reg[CURRENT_RS] );
	
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )   ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = (_u64)(((_u32)MipsCpu.GPR[CURRENT_RT].lo)  >>  ( MipsCpu.GPR[CURRENT_RS].lo & 0x1F));	
		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}



/*
Subtract Word
To subtract 32-bit integers. If overflow occurs, then trap
*/
void   SUB () {
	_u64 temp;

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sub	%s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo )  && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) { 
	
		temp = (_u64)((_s32)MipsCpu.GPR[CURRENT_RS].lo - (_s32)MipsCpu.GPR[CURRENT_RT].lo);	
		
		if ( IsNot32bitIntegerOverflow ( temp ) ) {
			//sign_extended(temp);
			MipsCpu.GPR[CURRENT_RD].lo = temp;
		}
		else {
			//SignalExeption(IntegerOverflow)
		}
	}
	
	INCREMENT_PC();
}



/*
Subtract Unsigned Word 
To subtract 32-bit integers
*/
void   SUBU   () {
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: subu %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );


	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo )  && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) ) { 
		
		MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_s32)MipsCpu.GPR[CURRENT_RS].lo - (_s32)MipsCpu.GPR[CURRENT_RT].lo);
//		sign_extended(MipsCpu.GPR[CURRENT_RD].lo);
	
	}
	
	INCREMENT_PC();
}



/*
Store Word
To store a word to memory
*/
void    SW   () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sd %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  MipsCpu.GPR[CURRENT_BASE].lo , (_s16)CURRENT_OFFSET ) ;

    INCREMENT_PC();
}




/*
Store Word Left
To store the most-significant part of a word to an unaligned memory address
MIPS32
*/

void    SWL  () {

    _u128   temp128;
    _u64    ReadAdress;

	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: swl %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );


	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xF)
	{
		case 0:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ( (MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 1:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 2:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 3:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 0 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 0 ) ;
		break;

		case 4:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 5:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 6:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 7:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 4 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 4 ) ;
		break;

		case 8:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 9:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 10:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 11:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 8 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 8 ) ;
		break;

		case 12:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 24)|(temp128.lo & 0xFFFFFFFFFFFFFF00);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 13:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 16)|(temp128.lo & 0xFFFFFFFFFFFF0000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 14:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) >> 8)|(temp128.lo & 0xFFFFFFFFFF000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;

		case 15:
			ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
			MipsMemoryRead ( &temp128 , DATA_DW ,  ReadAdress , 12 ) ;
			temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo & 0xFFFFFFFF) )|(temp128.lo & 0xFFFFFFFF00000000);
			MipsMemoryWrite ( temp128 , DATA_DW ,  ReadAdress , 12 ) ;
		break;


	}

	INCREMENT_PC();
}




/*
Store Word Right
To store the least-significant part of a word to an unaligned memory address
MIPS32
*/

void    SWR  () {
    _u128   temp128;
    _u64    ReadAdress;


	_u64 off;
	off = Extractoffset();
	sign_extendedh(off);

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sdr %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],(long)CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	switch ((MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xF)
	{
	case 0:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 1:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 2:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 3:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 0 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 0 ) ;
	break;

	case 4:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 5:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 6:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 7:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 4 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 4 ) ;
	break;

	case 8:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 9:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 10:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 11:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 8 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 8 ) ;
	break;

	case 12:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) ;
		MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_W ,  ReadAdress , 12 ) ;
	break;

	case 13:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 8) & 0xFFFFFF00) | (temp128.lo & 0x000000FF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 12 ) ;
	break;

	case 14:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 16) & 0xFFFF0000) | (temp128.lo & 0x0000FFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 12 ) ;
	break;

	case 15:
		ReadAdress = (MipsCpu.GPR[CURRENT_BASE].lo + off) & 0xFFFFFFF0 ;
		MipsMemoryRead ( &temp128 , DATA_W ,  ReadAdress , 12 ) ;
		temp128.lo = ((MipsCpu.GPR[CURRENT_RT].lo << 24) & 0xFF000000) | (temp128.lo & 0x00FFFFFF);
		MipsMemoryWrite ( temp128 , DATA_W ,  ReadAdress , 12 ) ;
	break;

	

	}

	INCREMENT_PC();
}




/*
Synchronize Shared Memory
To order loads and stores.
*/
void   SYNC  () {
    if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: SYNC", (long)MipsCpu.PC);
    INCREMENT_PC();
}


/*
System Call
To cause a System Call exception
*/
void   SYSCALL() {
	_u8		SystemCallNumber;	
 	
  	SystemCallNumber = MipsCpu.GPR[CPU_REG_V1].lo & 0xFF;
	
 	if ((signed char)SystemCallNumber < 0) SystemCallNumber = (- SystemCallNumber ) & 0xFF ;
 
 	syscall_fct[SystemCallNumber]();
}



void   TEQ() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TEQI() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void   TGE() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TGEI() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TGEIU() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TGEU() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TLT() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TLTI() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TLTIU() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TLTU() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TNE() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void   TNEI() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

/*
Xor
To do a bitwise logical XOR
*/
void   XOR  () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: or %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );

	
	MipsCpu.GPR[CURRENT_RD].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo ^ (_u64)MipsCpu.GPR[CURRENT_RT].lo);
	
	INCREMENT_PC();
}



/*
Exclusive OR Immediate
To do a bitwise logical Exclusive OR with a constant
*/
void    XORI () {

	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: xori %s,%s,%4llx", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],debug_r5900_reg[CURRENT_RS],CURRENT_IMM );
	
	MipsCpu.GPR[CURRENT_RT].lo = (_u64)((_u64)MipsCpu.GPR[CURRENT_RS].lo ^ (_u64)CURRENT_IMM);
	
	INCREMENT_PC();
}








/*****************************************************************************************************************
									UNCLASSIFIED INSTRUCTION SET
*****************************************************************************************************************/

/*
Perform Cache Operation
To perform the cache operation specified by op.
MIPS32
*/
void    CACHE( ) {
	if ( debug_op == 1 )   sprintf(debugTempText,"%8.8llx: CACHE", (long)MipsCpu.PC);
    
    INCREMENT_PC();
}




void	CVT_D()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}			



void	MOVF_fmt()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void	MOVT_fmt()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}	



