/**
 * nSX2 - an OpenSource PS2 Emulator
 * Copyright (C) 2002 Muad
 *
 * homepage: http://nsx2.emulation64.com
 * email address: atreides.muaddib@caramail.com
 * 
 * If you want to contribute to the project please contact
 * me first (maybe someone is already making what you are
 * planning to do).
 *
 *
 * This program is free software; you can redistribute it and/
 * or modify it under the terms of the GNU General Public Li-
 * cence as published by the Free Software Foundation; either
 * version 2 of the Licence, or any later version.
 *
 * This program is distributed in the hope that it will be use-
 * ful, but WITHOUT ANY WARRANTY; without even the implied war-
 * ranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public Licence for more details.
 *
 * You should have received a copy of the GNU General Public
 * Licence along with this program; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139,
 * USA.
 *
**/
#include <stdio.h>
#include <stdlib.h>

#include <strings.h>
#include <float.h>
#include <math.h>

#include "nSX2.h"
 
#include "r5900_main.h"
#include "r5900_main_table.h"

#include "PS2Edefs.h"


/*****************************************************************************************************************
								EE SPECIFIC INSTRUCTION SET
*****************************************************************************************************************/

void    SPECIAL2()	{	special2_opc[(_OPCODE_  & 0x3F)](); }

void 	MMI0()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}
 
void 	MMI2()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	MMI1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	MMI3()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void   DIV1  () {		

	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: div1 %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) && ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   
	    MipsCpu.GPR[CPU_REG_LO].hi = (_s32)MipsCpu.GPR[CURRENT_RS].lo / (_s32)MipsCpu.GPR[CURRENT_RT].lo;
        MipsCpu.GPR[CPU_REG_LO].hi = (_s32)MipsCpu.GPR[CURRENT_RS].lo % (_s32)MipsCpu.GPR[CURRENT_RT].lo;
		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].hi);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].hi);
		
	}
	
	INCREMENT_PC();
}
 

void 	DIVU1()		{	
	
	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: divu1 %s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);

    if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo ) && ( MipsCpu.GPR[CURRENT_RT].lo != 0 )  ) {   

        MipsCpu.GPR[CPU_REG_LO].hi = (_u32)MipsCpu.GPR[CURRENT_RS].lo / (_u32)MipsCpu.GPR[CURRENT_RT].lo;
        MipsCpu.GPR[CPU_REG_LO].hi = (_u32)MipsCpu.GPR[CURRENT_RS].lo % (_u32)MipsCpu.GPR[CURRENT_RT].lo;
		
		sign_extended( MipsCpu.GPR[CPU_REG_LO].hi);
		sign_extended( MipsCpu.GPR[CPU_REG_HI].hi);
	}
	
	INCREMENT_PC();
} 


void    LQ()		{	
	_u64 off;

	off = Extractoffset();
	sign_extendedh(off);
	
	if ( debug_op == 1 ) 		sprintf(debugTempText,"%8.8llx: lq %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryRead ( &MipsCpu.GPR[CURRENT_RT] , DATA_QUAD ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;

    INCREMENT_PC();
}


void	MADD()		{	
	_s64 temp;

	if ( debug_op == 1 )  	sprintf(debugTempText,"%8.8llx: madd %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD],debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT]);
  
	if ( IsSignExtended( MipsCpu.GPR[CURRENT_RS].lo ) && IsSignExtended( MipsCpu.GPR[CURRENT_RT].lo )  ) {

    temp = (_s64)((u64)MipsCpu.GPR[CPU_REG_LO].lo | ((u64)MipsCpu.GPR[CPU_REG_HI].lo << 32)) + 
               ((_s64)MipsCpu.GPR[CURRENT_RS].lo * (_s64)MipsCpu.GPR[CURRENT_RT].lo);
	
		MipsCpu.GPR[CPU_REG_LO].lo = (_s32)( temp  );
		MipsCpu.GPR[CPU_REG_HI].lo = (_s32)( (_s64)temp  >> 32 );


		if ( CURRENT_RD != 0 ) MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].lo;

	}

	INCREMENT_PC();

} 


void	MADD1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 
		
void	MADDU()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void	MADDU1()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	MFHI1()		{	

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mfhi1 %s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_HI].hi;

    INCREMENT_PC();
}

/*
*/
void 	MFLO1()		{	

	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mflo1 %s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD]  );
	
	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].hi;

    INCREMENT_PC();


}




void 	MFSA()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void 	MTSA()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void 	MTSAB() 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void 	MTSAH () 	{	PrintDebugOpcode() ; MipsCpu.halt = 1;}


void	MULT1()	{
	_s64 temp;
	
	if ( debug_op == 1 ) sprintf(debugTempText,"%8.8llx: mult1 %s,%s,%s", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RD] , debug_r5900_reg[CURRENT_RS],debug_r5900_reg[CURRENT_RT] );
	
	temp = (_s64)((_s32)(MipsCpu.GPR[CURRENT_RS].lo )) * ((_s32)( MipsCpu.GPR[CURRENT_RT].lo ));
	
	MipsCpu.GPR[CPU_REG_LO].hi = (_u32)temp;
	MipsCpu.GPR[CPU_REG_HI].hi = (_u32)((_u64)temp  >> 32 );

	MipsCpu.GPR[CURRENT_RD].lo = MipsCpu.GPR[CPU_REG_LO].hi ;
	
	INCREMENT_PC();
}



void 	MULTU1()	{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void	PLZCW()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PMFHL()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PMTHL()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	MTHI1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;}

void 	MTLO1()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PSLLH()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PSLLW()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 
	
void 	PSRLH()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PSRAH()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PSRLW()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	PSRAW()		{	PrintDebugOpcode() ; MipsCpu.halt = 1;} 

void 	SQ() 		{

	_u64 off;
	
	off = Extractoffset();
	sign_extendedh(off);
	
	if ( debug_op == 1 ) 	sprintf(debugTempText,"%8.8llx: sq %s,%4llx(%s)", (long)MipsCpu.PC, debug_r5900_reg[CURRENT_RT],CURRENT_OFFSET,debug_r5900_reg[CURRENT_BASE] );

	MipsMemoryWrite ( MipsCpu.GPR[CURRENT_RT] , DATA_QUAD ,  MipsCpu.GPR[CURRENT_BASE].lo , off ) ;
    INCREMENT_PC();
}




